# Copyright (C) 2013 Riverbank Computing Limited.
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from argparse import ArgumentParser, RawTextHelpFormatter
import sys

from PySide6.QtCore import (QByteArray, QFile, QFileInfo, QSaveFile, QSettings,
                            QTextStream, Qt, Slot)
from PySide6.QtGui import QAction, QIcon, QKeySequence
from PySide6.QtWidgets import (QApplication, QFileDialog, QMainWindow,
                               QMessageBox, QTextEdit)

import application_rc  # noqa: F401


class MainWindow(QMainWindow):
    def __init__(self):
        super().__init__()

        self._cur_file = ''

        self._text_edit = QTextEdit()
        self.setCentralWidget(self._text_edit)

        self.create_actions()
        self.create_menus()
        self.create_tool_bars()
        self.create_status_bar()

        self.read_settings()

        self._text_edit.document().contentsChanged.connect(self.document_was_modified)

        self.set_current_file('')
        self.setUnifiedTitleAndToolBarOnMac(True)

    def closeEvent(self, event):
        if self.maybe_save():
            self.write_settings()
            event.accept()
        else:
            event.ignore()

    @Slot()
    def new_file(self):
        if self.maybe_save():
            self._text_edit.clear()
            self.set_current_file('')

    @Slot()
    def open(self):
        if self.maybe_save():
            fileName, filtr = QFileDialog.getOpenFileName(self)
            if fileName:
                self.load_file(fileName)

    @Slot()
    def save(self):
        if self._cur_file:
            return self.save_file(self._cur_file)

        return self.save_as()

    @Slot()
    def save_as(self):
        fileName, filtr = QFileDialog.getSaveFileName(self)
        if fileName:
            return self.save_file(fileName)

        return False

    @Slot()
    def about(self):
        QMessageBox.about(self, "About Application",
                          "The <b>Application</b> example demonstrates how to write "
                          "modern GUI applications using Qt, with a menu bar, "
                          "toolbars, and a status bar.")

    @Slot()
    def document_was_modified(self):
        self.setWindowModified(self._text_edit.document().isModified())

    def create_actions(self):
        icon = QIcon.fromTheme(QIcon.ThemeIcon.DocumentNew, QIcon(':/images/new.png'))
        self._new_act = QAction(icon, "&New", self, shortcut=QKeySequence.StandardKey.New,
                                statusTip="Create a new file", triggered=self.new_file)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.DocumentOpen, QIcon(':/images/open.png'))
        self._open_act = QAction(icon, "&Open...", self,
                                 shortcut=QKeySequence.StandardKey.Open,
                                 statusTip="Open an existing file",
                                 triggered=self.open)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.DocumentSave, QIcon(':/images/save.png'))
        self._save_act = QAction(icon, "&Save", self,
                                 shortcut=QKeySequence.StandardKey.Save,
                                 statusTip="Save the document to disk", triggered=self.save)

        self._save_as_act = QAction("Save &As...", self,
                                    shortcut=QKeySequence.StandardKey.SaveAs,
                                    statusTip="Save the document under a new name",
                                    triggered=self.save_as)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.ApplicationExit)
        self._exit_act = QAction(icon, "E&xit", self, shortcut="Ctrl+Q",
                                 statusTip="Exit the application", triggered=self.close)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.EditCut, QIcon(':/images/cut.png'))
        self._cut_act = QAction(icon, "Cu&t", self, shortcut=QKeySequence.StandardKey.Cut,
                                statusTip="Cut the current selection's contents to the clipboard",
                                triggered=self._text_edit.cut)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.EditCopy, QIcon(':/images/copy.png'))
        self._copy_act = QAction(icon, "&Copy",
                                 self, shortcut=QKeySequence.StandardKey.Copy,
                                 statusTip="Copy the current selection's contents to the clipboard",
                                 triggered=self._text_edit.copy)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.EditPaste, QIcon(':/images/paste.png'))
        self._paste_act = QAction(icon, "&Paste",
                                  self, shortcut=QKeySequence.StandardKey.Paste,
                                  statusTip="Paste the clipboard's contents into the current "
                                  "selection",
                                  triggered=self._text_edit.paste)

        icon = QIcon.fromTheme(QIcon.ThemeIcon.HelpAbout)
        self._about_act = QAction(icon, "&About", self,
                                  statusTip="Show the application's About box",
                                  triggered=self.about)

        self._about_qt_act = QAction("About &Qt", self,
                                     statusTip="Show the Qt library's About box",
                                     triggered=qApp.aboutQt)  # noqa: F821

        self._cut_act.setEnabled(False)
        self._copy_act.setEnabled(False)
        self._text_edit.copyAvailable.connect(self._cut_act.setEnabled)
        self._text_edit.copyAvailable.connect(self._copy_act.setEnabled)

    def create_menus(self):
        self._file_menu = self.menuBar().addMenu("&File")
        self._file_menu.addAction(self._new_act)
        self._file_menu.addAction(self._open_act)
        self._file_menu.addAction(self._save_act)
        self._file_menu.addAction(self._save_as_act)
        self._file_menu.addSeparator()
        self._file_menu.addAction(self._exit_act)

        self._edit_menu = self.menuBar().addMenu("&Edit")
        self._edit_menu.addAction(self._cut_act)
        self._edit_menu.addAction(self._copy_act)
        self._edit_menu.addAction(self._paste_act)

        self.menuBar().addSeparator()

        self._help_menu = self.menuBar().addMenu("&Help")
        self._help_menu.addAction(self._about_act)
        self._help_menu.addAction(self._about_qt_act)

    def create_tool_bars(self):
        self._file_tool_bar = self.addToolBar("File")
        self._file_tool_bar.addAction(self._new_act)
        self._file_tool_bar.addAction(self._open_act)
        self._file_tool_bar.addAction(self._save_act)

        self._edit_tool_bar = self.addToolBar("Edit")
        self._edit_tool_bar.addAction(self._cut_act)
        self._edit_tool_bar.addAction(self._copy_act)
        self._edit_tool_bar.addAction(self._paste_act)

    def create_status_bar(self):
        self.statusBar().showMessage("Ready")

    def read_settings(self):
        settings = QSettings('QtProject', 'Application Example')
        geometry = settings.value('geometry', QByteArray())
        if geometry.size():
            self.restoreGeometry(geometry)

    def write_settings(self):
        settings = QSettings('QtProject', 'Application Example')
        settings.setValue('geometry', self.saveGeometry())

    def maybe_save(self):
        if self._text_edit.document().isModified():
            ret = QMessageBox.warning(self, "Application",
                                      "The document has been modified.\nDo you want to save "
                                      "your changes?",
                                      QMessageBox.StandardButton.Save
                                      | QMessageBox.StandardButton.Discard
                                      | QMessageBox.StandardButton.Cancel)
            if ret == QMessageBox.StandardButton.Save:
                return self.save()
            elif ret == QMessageBox.StandardButton.Cancel:
                return False
        return True

    def load_file(self, fileName):
        file = QFile(fileName)
        if not file.open(QFile.OpenModeFlag.ReadOnly | QFile.OpenModeFlag.Text):
            reason = file.errorString()
            QMessageBox.warning(self, "Application", f"Cannot read file {fileName}:\n{reason}.")
            return

        inf = QTextStream(file)
        with QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor):
            self._text_edit.setPlainText(inf.readAll())

        self.set_current_file(fileName)
        self.statusBar().showMessage("File loaded", 2000)

    def save_file(self, fileName):
        error = None
        with QApplication.setOverrideCursor(Qt.CursorShape.WaitCursor):
            file = QSaveFile(fileName)
            if file.open(QFile.OpenModeFlag.WriteOnly | QFile.OpenModeFlag.Text):
                outf = QTextStream(file)
                outf << self._text_edit.toPlainText()
                if not file.commit():
                    reason = file.errorString()
                    error = f"Cannot write file {fileName}:\n{reason}."
            else:
                reason = file.errorString()
                error = f"Cannot open file {fileName}:\n{reason}."

        if error:
            QMessageBox.warning(self, "Application", error)
            return False

        self.set_current_file(fileName)
        self.statusBar().showMessage("File saved", 2000)
        return True

    def set_current_file(self, fileName):
        self._cur_file = fileName
        self._text_edit.document().setModified(False)
        self.setWindowModified(False)

        if self._cur_file:
            shown_name = self.stripped_name(self._cur_file)
        else:
            shown_name = 'untitled.txt'

        self.setWindowTitle(f"{shown_name}[*] - Application")

    def stripped_name(self, fullFileName):
        return QFileInfo(fullFileName).fileName()


if __name__ == '__main__':
    argument_parser = ArgumentParser(description='Application Example',
                                     formatter_class=RawTextHelpFormatter)
    argument_parser.add_argument("file", help="File",
                                 nargs='?', type=str)
    options = argument_parser.parse_args()

    app = QApplication(sys.argv)
    main_win = MainWindow()
    if options.file:
        main_win.load_file(options.file)
    main_win.show()
    sys.exit(app.exec())



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QLockFile'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
sys.path.append(os.fspath(Path(__file__).resolve().parents[1] / "util"))
from init_paths import init_test_paths
init_test_paths()

from PySide6.QtCore import QDir, QLockFile, QCoreApplication


class TestQMessageAuthenticationCode (unittest.TestCase):

    def setUp(self):
        pid = QCoreApplication.applicationPid()
        self._fileName = f"{QDir.tempPath()}/pqlockfiletest{pid}.tmp"

    def tearDown(self):
        if (os.path.exists(self._fileName)):
            os.remove(self._fileName)

    def test(self):
        # Merely exercise the API, no locking against another processes.
        lockFile = QLockFile(self._fileName)
        self.assertTrue(lockFile.lock())
        self.assertTrue(lockFile.isLocked())
        lock_info = lockFile.getLockInfo()
        self.assertEqual(len(lock_info), 3)
        self.assertEqual(lock_info[0], os.getpid())
        lockFile.unlock()


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for implicit conversion generated by conversion operator.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Time, StrList


class ConversionOperatorTest(unittest.TestCase):
    '''Test cases for implicit conversion generated by conversion operator.'''

    def testConversionOperator(self):
        '''Time defined an conversion operator for Str, so passing a Time object
           to a method expecting a Str should work.'''
        t = Time(1, 2, 3)
        t_str = t.toString()
        sl = StrList()

        # StrList.append expects a Str object.
        sl.append(t)

        self.assertEqual(len(sl), 1)
        self.assertEqual(sl[0], t_str)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for std::pair container conversions'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import PairUser


class ExtendedPairUser(PairUser):
    def __init__(self):
        PairUser.__init__(self)
        self.create_pair_called = False

    def createPair(self):
        self.create_pair_called = True
        return (7, 13)


class PairConversionTest(unittest.TestCase):
    '''Test case for std::pair container conversions'''

    def testReimplementedVirtualMethodCall(self):
        '''Test if a Python override of a virtual method is correctly called from C++.'''
        pu = ExtendedPairUser()
        pair = pu.callCreatePair()
        self.assertTrue(pu.create_pair_called)
        self.assertEqual(type(pair), tuple)
        self.assertEqual(type(pair[0]), int)
        self.assertEqual(type(pair[1]), int)
        self.assertEqual(pair, (7, 13))

    def testPrimitiveConversionInsideContainer(self):
        '''Test primitive type conversion inside conversible std::pair container.'''
        cpx0 = complex(1.2, 3.4)
        cpx1 = complex(5.6, 7.8)
        cp = PairUser.createComplexPair(cpx0, cpx1)
        self.assertEqual(type(cp), tuple)
        self.assertEqual(type(cp[0]), complex)
        self.assertEqual(type(cp[1]), complex)
        self.assertEqual(cp, (cpx0, cpx1))

    def testSumPair(self):
        '''Test method that sums the items of a pair using values of the types
           expected by C++ (int and double)'''
        pu = PairUser()
        pair = (3, 7.13)
        result = pu.sumPair(pair)
        self.assertEqual(result, sum(pair))

    def testSumPairDifferentTypes(self):
        '''Test method that sums the items of a pair using values of types different
           from the ones expected by C++ (int and double)'''
        pu = PairUser()
        pair = (3.3, 7)
        result = pu.sumPair(pair)
        self.assertNotEqual(result, sum(pair))
        self.assertEqual(result, int(pair[0]) + pair[1])

    def testConversionInBothDirections(self):
        '''Test converting a pair from Python to C++ and the other way around.'''
        pu = PairUser()
        pair = (3, 5)
        pu.setPair(pair)
        result = pu.getPair()
        self.assertEqual(result, pair)

    def testConversionInBothDirectionsWithSimilarContainer(self):
        '''Test converting a list, instead of the expected tuple, from Python to C++
           and the other way around.'''
        pu = PairUser()
        pair = [3, 5]
        pu.setPair(pair)
        result = pu.getPair()
        self.assertNotEqual(result, pair)
        self.assertEqual(result, tuple(pair))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Unit test for QOpenGLContext, QOpenGLTexture, QOpenGLWindow and related classes'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication

from PySide6.QtCore import QSize, QTimer, Qt
from PySide6.QtGui import QColor, QImage, QSurfaceFormat
from PySide6.QtOpenGL import QOpenGLTexture, QOpenGLWindow


try:
    from OpenGL import GL
    from PySide6.QtOpenGL import QOpenGLVersionProfile, QOpenGLVersionFunctionsFactory
except ImportError:
    print("Skipping test due to missing OpenGL module/GLES only build")
    sys.exit(0)


class OpenGLWindow(QOpenGLWindow):
    def __init__(self):
        super().__init__()

        self.m_functions = None
        self.m_texture = None
        self.visibleChanged.connect(self.slotVisibleChanged)

    def slotVisibleChanged(self, visible):
        if not visible and self.m_texture is not None and self.context().makeCurrent(self):
            self.m_texture = None
            self.context().doneCurrent()

    def initializeGL(self):
        profile = QOpenGLVersionProfile()
        profile.setVersion(1, 3)
        profile.setProfile(QSurfaceFormat.OpenGLContextProfile.CompatibilityProfile)
        self.m_functions = QOpenGLVersionFunctionsFactory.get(profile)
        self.m_functions.initializeOpenGLFunctions()

        print("GL_MAX_LIGHTS=", self.m_functions.glGetIntegerv(GL.GL_MAX_LIGHTS))
        image = QImage(QSize(200, 200), QImage.Format.Format_RGBA8888)
        image.fill(QColor(Qt.GlobalColor.red))
        self.m_texture = QOpenGLTexture(image)

    def paintGL(self):
        self.m_functions.glMatrixMode(GL.GL_MODELVIEW)
        self.m_functions.glLoadIdentity()

        self.m_functions.glMatrixMode(GL.GL_PROJECTION)
        self.m_functions.glLoadIdentity()
        self.m_functions.glOrtho(0, 1, 1, 0, -1, 1)

        self.m_functions.glClear(GL.GL_COLOR_BUFFER_BIT)
        self.m_functions.glEnable(GL.GL_TEXTURE_2D)
        self.m_texture.bind()

        d = 0.5
        self.m_functions.glBegin(GL.GL_QUADS)
        self.m_functions.glTexCoord2f(0, 0)
        self.m_functions.glVertex2f(0, 0)
        self.m_functions.glTexCoord2f(d, 0)
        self.m_functions.glVertex2f(d, 0)
        self.m_functions.glTexCoord2f(d, d)
        self.m_functions.glVertex2f(d, d)
        self.m_functions.glTexCoord2f(0, d)
        self.m_functions.glVertex2f(0, d)
        self.m_functions.glEnd()
        self.m_texture.release()

    def resizeGL(self, w, h):
        self.m_functions.glViewport(0, 0, self.width(), self.height())


class QOpenGLWindowTest(UsesQApplication):
    # On macOS, glClear(), glViewport() are rejected due to
    # GLbitfield/GLint not being resolved properly
    def test(self):
        openGlWindow = OpenGLWindow()
        openGlWindow.resize(640, 480)
        openGlWindow.show()
        QTimer.singleShot(100, openGlWindow.close)
        self.app.exec()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtGui import QMatrix4x4


class TestQMatrix(unittest.TestCase):
    def testOperator(self):
        m = QMatrix4x4(1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16)
        v = 1
        for x in range(4):
            for y in range(4):
                self.assertEqual(m[x, y], v)
                v += 1


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# Copyright (C) 2011 Thomas Perl <m@thp.io>
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

# Test case for PySide bug 814
# http://bugs.pyside.org/show_bug.cgi?id=814
# archive:
# https://srinikom.github.io/pyside-bz-archive/814.html
# 2011-04-08 Thomas Perl <m@thp.io>
# Released under the same terms as PySide itself

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.helper import quickview_errorstring
from helper.timedqguiapplication import TimedQGuiApplication

from PySide6.QtCore import QUrl, QAbstractListModel, QModelIndex, Qt
from PySide6.QtQuick import QQuickView
from PySide6.QtQml import QmlElement

QML_IMPORT_NAME = "test.ListModel"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class ListModel(QAbstractListModel):
    def __init__(self):
        super().__init__()

    def roleNames(self):
        return {Qt.ItemDataRole.DisplayRole: b'pysideModelData'}

    def rowCount(self, parent=QModelIndex()):
        return 3

    def data(self, index, role):
        if index.isValid() and role == Qt.ItemDataRole.DisplayRole:
            return 'blubb'
        return None


class TestBug814(TimedQGuiApplication):
    def testAbstractItemModelTransferToQML(self):
        view = QQuickView()
        model = ListModel()
        view.setInitialProperties({"pythonModel": model})
        file = Path(__file__).resolve().parent / 'bug_814.qml'
        self.assertTrue(file.is_file())
        view.setSource(QUrl.fromLocalFile(file))
        root = view.rootObject()
        self.assertTrue(root, quickview_errorstring(view))
        view.show()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import QObject, Property, Signal
from PySide6.QtGui import QColor
from PySide6.QtQml import QmlAnonymous, QmlElement

# To be used on the @QmlElement decorator
# (QML_IMPORT_MINOR_VERSION is optional)
QML_IMPORT_NAME = "People"
QML_IMPORT_MAJOR_VERSION = 1


@QmlAnonymous
class ShoeDescription(QObject):
    brand_changed = Signal()
    size_changed = Signal()
    price_changed = Signal()
    color_changed = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self._brand = ''
        self._size = 0
        self._price = 0
        self._color = QColor()

    @Property(str, notify=brand_changed, final=True)
    def brand(self):
        return self._brand

    @brand.setter
    def brand(self, b):
        if self._brand != b:
            self._brand = b
            self.brand_changed.emit()

    @Property(int, notify=size_changed, final=True)
    def size(self):
        return self._size

    @size.setter
    def size(self, s):
        if self._size != s:
            self._size = s
            self.size_changed.emit()

    @Property(float, notify=price_changed, final=True)
    def price(self):
        return self._price

    @price.setter
    def price(self, p):
        if self._price != p:
            self._price = p
            self.price_changed.emit()

    @Property(QColor, notify=color_changed, final=True)
    def color(self):
        return self._color

    @color.setter
    def color(self, c):
        if self._color != c:
            self._color = c
            self.color_changed.emit()


@QmlAnonymous
class Person(QObject):
    name_changed = Signal()

    def __init__(self, parent=None):
        super().__init__(parent)
        self._name = ''
        self._shoe = ShoeDescription()

    @Property(str, notify=name_changed, final=True)
    def name(self):
        return self._name

    @name.setter
    def name(self, n):
        if self._name != n:
            self._name = n
            self.name_changed.emit()

    @Property(ShoeDescription, final=True)
    def shoe(self):
        return self._shoe


@QmlElement
class Boy(Person):
    def __init__(self, parent=None):
        super().__init__(parent)


@QmlElement
class Girl(Person):
    def __init__(self, parent=None):
        super().__init__(parent)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for multiple inheritance in mixed Python/C++ scenarios'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import ObjectType


class Base:
    '''Base Python class'''

    def __init__(self):
        self.name = ''

    def pythonName(self):
        return self.name

    def setPythonName(self, name):
        self.name = name


class Child(Base, ObjectType):
    '''Dummy class with mixed parents'''

    def __init__(self):
        Base.__init__(self)
        ObjectType.__init__(self)


class MixedInheritanceTest(unittest.TestCase):

    def testMixed(self):
        '''Mixed Python/C++ multiple inheritance'''
        obj = Child()

        obj.setObjectName('aaa')
        self.assertEqual(obj.objectName(), 'aaa')

        obj.setPythonName('python')
        self.assertEqual(obj.pythonName(), 'python')


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2018 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
from __future__ import annotations

import os
import sys
from pathlib import Path
from typing import Any

from . import PYPROJECT_PATH, PYSIDE, PYSIDE_MODULE, SHIBOKEN
from .log import LogLevel, log
from .utils import available_pyside_tools, Singleton

try:
    import tomllib
except ModuleNotFoundError:
    import tomli as tomllib


class Config(metaclass=Singleton):
    def __init__(self):
        # Constants
        self._build_type_all = "all"
        self._invocation_type_top_level = "top-level"
        self._invocation_type_internal = "internal"

        # The keyword arguments which will be given to setuptools.setup
        self.setup_kwargs = {}

        # The setup.py invocation type.
        # top-level
        # internal
        self.invocation_type: str = ""

        # The type of the top-level build.
        # all - build shiboken6 module, shiboken6-generator and PySide6
        #       modules
        # shiboken6 - build only shiboken6 module
        # shiboken6-generator - build only the shiboken6-generator
        # pyside6 - build only PySide6 modules
        self.build_type: str = ""

        # The internal build type, used for internal invocations of
        # setup.py to build a specific module only.
        self.internal_build_type: str = ""

        # Options that can be given to --build-type and
        # --internal-build-type
        self.shiboken_module_option_name = SHIBOKEN
        self.shiboken_generator_option_name = f"{SHIBOKEN}-generator"
        self.pyside_option_name = PYSIDE

        # Names to be passed to setuptools.setup() name key,
        # so not package name, but rather project name as it appears
        # in the wheel name and on PyPi.
        self.shiboken_module_st_name: str = SHIBOKEN
        self.shiboken_generator_st_name: str = f"{SHIBOKEN}-generator"
        self.pyside_st_name: str = PYSIDE_MODULE

        # Path to CMake toolchain file when intending to cross compile
        # the project.
        self.cmake_toolchain_file: str | os.PathLike = ""

        # Store where host shiboken is built during a cross-build.
        self.shiboken_host_query_path: str = ""

        self.setup_script_dir: str | os.PathLike = ""

        # Getting data from base pyproject.toml file to be consistent

        if not PYPROJECT_PATH.exists():
            log.error("'pyproject.toml.base' not found in '{pyproject_path}'")

        with open(PYPROJECT_PATH, "rb") as f:
            _pyproject_data = tomllib.load(f)["project"]

        self.setup_kwargs: dict[str, Any] = {}
        self.setup_kwargs['long_description_content_type'] = 'text/markdown'

        self.setup_kwargs['keywords'] = _pyproject_data["keywords"]
        _author = _pyproject_data["authors"][0]
        self.setup_kwargs['author'] = _author["name"]
        self.setup_kwargs['author_email'] = _author["email"]
        self.setup_kwargs['url'] = _pyproject_data["urls"]["Homepage"]
        self.setup_kwargs['license'] = _pyproject_data["license"]["text"]
        self.setup_kwargs['python_requires'] = _pyproject_data["requires-python"]

        self.classifiers = _pyproject_data["classifiers"]
        self.setup_kwargs['classifiers'] = self.classifiers

    def init_config(self,
                    build_type="",
                    internal_build_type="",
                    cmd_class_dict=None,
                    package_version=None,
                    ext_modules=None,
                    setup_script_dir: str | os.PathLike = "",
                    cmake_toolchain_file: str | os.PathLike = "",
                    log_level=LogLevel.INFO,
                    qt_install_dir: str | os.PathLike = ""):
        """
        Sets up the global singleton config which is used in many parts
        of the setup process.
        """

        # if --internal-build-type was passed, it means that this is a
        # sub-invocation to build a specific package.
        if internal_build_type:
            self.set_is_internal_invocation()
            self.set_internal_build_type(internal_build_type)
        else:
            self.set_is_top_level_invocation()

        # --build-type was specified explicitly, so set it. Otherwise
        # default to all.
        if build_type:
            self.build_type = build_type
        else:
            self.build_type = self._build_type_all

        self.setup_script_dir = Path(setup_script_dir)

        self.cmake_toolchain_file = cmake_toolchain_file

        self.setup_kwargs['long_description'] = self.get_long_description()
        self.setup_kwargs['cmdclass'] = cmd_class_dict
        self.setup_kwargs['version'] = package_version

        if log_level == LogLevel.QUIET:
            # Tells setuptools to be quiet, and only print warnings or errors.
            # Makes way less noise in the terminal when building.
            self.setup_kwargs['verbose'] = 0

        # Setting these two keys is still a bit of a discussion point.
        # In general not setting them will allow using "build" and
        # "bdist_wheel" just fine. What they do, is they specify to the
        # setuptools.command.build_py command that certain pure python
        # modules (.py files) exist in the specified package location,
        # and that they should be copied over to the setuptools build
        # dir.
        # But it doesn't really make sense for us, because we copy all
        # the necessary files to the build dir via prepare_packages()
        # function anyway.
        # If we don't set them, the build_py sub-command will be
        # skipped, but the build command will still be executed, which
        # is where we run cmake / make.
        # The only plausible usage of it, is if we will implement a
        # correctly functioning setup.py develop command (or bdist_egg).
        # But currently that doesn't seem to work.
        self.setup_kwargs['packages'] = self.get_setup_tools_packages_for_current_build()
        self.setup_kwargs['package_dir'] = self.get_package_name_to_dir_path_mapping()

        # Add a bogus extension module (will never be built here since
        # we are overriding the build command to do it using cmake) so
        # things like bdist_egg will know that there are extension
        # modules and will name the dist with the full platform info.
        self.setup_kwargs['ext_modules'] = ext_modules

        package_name = self.package_name()

        if self.internal_build_type == self.shiboken_module_option_name:
            self.setup_kwargs['name'] = self.shiboken_module_st_name
            self.setup_kwargs['description'] = "Python / C++ bindings helper module"
            self.setup_kwargs['entry_points'] = {}

        elif self.internal_build_type == self.shiboken_generator_option_name:
            self.setup_kwargs['name'] = self.shiboken_generator_st_name
            self.setup_kwargs['description'] = "Python / C++ bindings generator"
            self.setup_kwargs['install_requires'] = [
                f"{self.shiboken_module_st_name}=={package_version}"
            ]
            self.setup_kwargs['entry_points'] = {
                'console_scripts': [
                    f'{SHIBOKEN} = {package_name}.scripts.shiboken_tool:main',
                    f'{SHIBOKEN}-genpyi = {package_name}.scripts.shiboken_tool:genpyi',
                ]
            }

        elif self.internal_build_type == self.pyside_option_name:
            self.setup_kwargs['name'] = self.pyside_st_name
            self.setup_kwargs['description'] = (
                "Python bindings for the Qt cross-platform application and UI framework"
            )
            self.setup_kwargs['install_requires'] = [
                f"{self.shiboken_module_st_name}=={package_version}"
            ]
            if qt_install_dir:
                _pyside_tools = available_pyside_tools(qt_tools_path=Path(qt_install_dir))

                # replacing pyside6-android_deploy by pyside6-android-deploy for consistency
                # Also, the tool should not exist in any other platform than Linux and macOS
                _console_scripts = []
                if ("android_deploy" in _pyside_tools) and sys.platform in ["linux", "darwin"]:
                    _console_scripts = [(f"{PYSIDE}-android-deploy ="
                                        " PySide6.scripts.pyside_tool:android_deploy")]
                _pyside_tools.remove("android_deploy")

                _console_scripts.extend([f'{PYSIDE}-{tool} = {package_name}.scripts.pyside_tool:'
                                         f'{tool}' for tool in _pyside_tools])

                self.setup_kwargs['entry_points'] = {'console_scripts': _console_scripts}

    def get_long_description(self):
        readme_filename = 'README.md'
        changes_filename = 'CHANGES.rst'

        if self.is_internal_shiboken_module_build():
            readme_filename = f'README.{SHIBOKEN}.md'
        elif self.is_internal_shiboken_generator_build():
            readme_filename = f'README.{SHIBOKEN}-generator.md'
        elif self.is_internal_pyside_build():
            readme_filename = f'README.{PYSIDE}.md'

        with open(Path(self.setup_script_dir) / readme_filename) as f:
            readme = f.read()

        # Don't include CHANGES.rst for now, because we have not decided
        # how to handle change files yet.
        include_changes = False
        if include_changes:
            try:
                changes = ''
                with open(Path(self.setup_script_dir) / changes_filename) as f:
                    changes = f.read()
            except Exception as e:
                log.error(f"Couldn't read contents of {changes_filename}. {e}")
                raise
            return f"{readme}\n\n{changes}"

        return readme

    def package_name(self):
        """
        Returns package name as it appears in Python's site-packages
        directory.

        Package names can only be delimited by underscores, and not by
        dashes.
        """
        if self.is_internal_shiboken_module_build():
            return SHIBOKEN
        elif self.is_internal_shiboken_generator_build():
            return f"{SHIBOKEN}_generator"
        elif self.is_internal_pyside_build():
            return PYSIDE_MODULE
        else:
            return None

    def get_setup_tools_packages_for_current_build(self):
        """
        Returns a list of packages for setup tools to consider in the
        build_py command, so that it can copy the pure python files.
        Not really necessary because it's done in prepare_packages()
        anyway.

        This is really just to satisfy some checks in setuptools
        build_py command, and if we ever properly implement the develop
        command.
        """
        if self.internal_build_type == self.pyside_option_name:
            return [
                config.package_name(),
            ]
        elif self.internal_build_type == self.shiboken_module_option_name:
            return [self.package_name()]
        else:
            return []

    def get_package_name_to_dir_path_mapping(self):
        """
        Used in setuptools.setup 'package_dir' argument to specify where
        the actual module packages are located.

        For example when building the shiboken module, setuptools will
        expect to find the "shiboken6" module sources under
        "sources/{SHIBOKEN}/shibokenmodule".

        This is really just to satisfy some checks in setuptools
        build_py command, and if we ever properly implement the develop
        command.
        """
        if self.is_internal_shiboken_module_build():
            return {
                self.package_name(): f"sources/{SHIBOKEN}/shibokenmodule"
            }
        elif self.is_internal_shiboken_generator_build():
            # This is left empty on purpose, because the shiboken
            # generator doesn't have a python module for now.
            return {}
        elif self.is_internal_pyside_build():
            return {
                self.package_name(): f"sources/{PYSIDE}/{PYSIDE_MODULE}",
            }
        else:
            return {}

    def get_buildable_extensions(self):
        """
        Used by PysideBuild.run to build the CMake projects.
        :return: A list of directory names under the sources directory.
        """
        if self.is_internal_shiboken_module_build() or self.is_internal_shiboken_generator_build():
            return [SHIBOKEN]
        elif self.is_internal_pyside_build():
            return [PYSIDE, 'pyside-tools']
        return None

    def set_is_top_level_invocation(self):
        self.invocation_type = self._invocation_type_top_level

    def set_is_internal_invocation(self):
        self.invocation_type = self._invocation_type_internal

    def is_top_level_invocation(self):
        return self.invocation_type == self._invocation_type_top_level

    def is_internal_invocation(self):
        return self.invocation_type == self._invocation_type_internal

    def is_top_level_build_all(self):
        return self.build_type == self._build_type_all

    def is_top_level_build_shiboken_module(self):
        return self.build_type == self.shiboken_module_option_name

    def is_top_level_build_shiboken_generator(self):
        return self.build_type == self.shiboken_generator_option_name

    def is_top_level_build_pyside(self):
        return self.build_type == self.pyside_option_name

    def is_cross_compile(self):
        if not self.cmake_toolchain_file:
            return False
        return True

    def set_internal_build_type(self, internal_build_type):
        self.internal_build_type = internal_build_type

    def is_internal_shiboken_module_build(self):
        return self.internal_build_type == self.shiboken_module_option_name

    def is_internal_shiboken_generator_build(self):
        return self.internal_build_type == self.shiboken_generator_option_name

    def is_internal_pyside_build(self):
        return self.internal_build_type == self.pyside_option_name

    def is_internal_shiboken_generator_build_and_part_of_top_level_all(self):
        """
        Used to skip certain build rules and output, when we know that
        the CMake build of shiboken was already done as part of the
        top-level "all" build when shiboken6-module was built.
        """
        return self.is_internal_shiboken_generator_build() and self.is_top_level_build_all()

    def get_allowed_top_level_build_values(self):
        return [
            self._build_type_all,
            self.shiboken_module_option_name,
            self.shiboken_generator_option_name,
            self.pyside_option_name
        ]

    def get_allowed_internal_build_values(self):
        return [
            self.shiboken_module_option_name,
            self.shiboken_generator_option_name,
            self.pyside_option_name
        ]


config = Config()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""QUiLoader example, showing how to dynamically load a Qt Designer form
   from a UI file."""

from argparse import ArgumentParser, RawTextHelpFormatter
import sys

from PySide6.QtCore import QFile, QIODevice
from PySide6.QtWidgets import QApplication
from PySide6.QtUiTools import QUiLoader


if __name__ == '__main__':
    arg_parser = ArgumentParser(description="QUiLoader example",
                                formatter_class=RawTextHelpFormatter)
    arg_parser.add_argument('file', type=str, help='UI file')
    args = arg_parser.parse_args()
    ui_file_name = args.file

    app = QApplication(sys.argv)
    ui_file = QFile(ui_file_name)
    if not ui_file.open(QIODevice.OpenModeFlag.ReadOnly):
        reason = ui_file.errorString()
        print(f"Cannot open {ui_file_name}: {reason}")
        sys.exit(-1)
    loader = QUiLoader()
    widget = loader.load(ui_file, None)
    ui_file.close()
    if not widget:
        print(loader.errorString())
        sys.exit(-1)
    widget.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

from PySide6.QtCore import QAbstractTableModel, QTime, QTimer, Qt, Slot
from PySide6.QtWidgets import QApplication, QTableView

"""PySide6 port of the widgets/tutorials/modelview/3_changingmodel example from Qt v6.x"""


class MyModel(QAbstractTableModel):
#! [1]
    def __init__(self, parent=None):
        super().__init__(parent)
        self._timer = QTimer(self)
        self._timer.setInterval(1000)
        self._timer.timeout.connect(self.timer_hit)
        self._timer.start()
#! [1]

    def rowCount(self, parent=None):
        return 2

    def columnCount(self, parent=None):
        return 3

#! [2]
    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
        row = index.row()
        col = index.column()
        if role == Qt.ItemDataRole.DisplayRole and row == 0 and col == 0:
            return QTime.currentTime().toString()
        return None
#! [2]

#! [3]
    @Slot()
    def timer_hit(self):
        # we identify the top left cell
        top_left = self.createIndex(0, 0)
        # emit a signal to make the view reread identified data
        self.dataChanged.emit(top_left, top_left, [Qt.ItemDataRole.DisplayRole])
#! [3]


if __name__ == '__main__':
    app = QApplication(sys.argv)
    table_view = QTableView()
    my_model = MyModel()
    table_view.setModel(my_model)
    table_view.show()
    sys.exit(app.exec())



# Copyright (C) 2025 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause

from math import log10, sqrt
from PySide6.QtMultimedia import QAudioBuffer
from PySide6.QtWidgets import (QApplication, QHBoxLayout, QLabel, QSizePolicy, QToolButton,
                               QVBoxLayout, QWidget)
from PySide6.QtGui import QBrush, QPainter, QPalette
from PySide6.QtCore import QObject, QRectF, QThread, QTimer, qFuzzyCompare, Qt, Signal, Slot


# Constants used by AudioLevelMeter and MeterChannel
WIDGET_WIDTH = 34
MAX_CHANNELS = 8
PEAK_COLOR = "#1F9B5D"
RMS_COLOR = "#28C878"
RMS_WINDOW = 400  # ms
PEAK_LABEL_HOLD_TIME = 2000  # ms
DECAY_EASE_IN_TIME = 160  # ms
UPDATE_INTERVAL = 16  # ms, Assuming 60 Hz refresh rate.
DB_DECAY_PER_SECOND = 20.0
DB_DECAY_PER_UPDATE = DB_DECAY_PER_SECOND / (1000 / UPDATE_INTERVAL)
DB_MAX = 0.0
DB_MIN = -60.0


def amplitudeToDb(f):
    """Converts a float sample value to dB and clamps it between DB_MIN and DB_MAX."""
    if f <= 0:
        return DB_MIN
    v = 20.0 * log10(f)
    if v < DB_MIN:
        return DB_MIN
    if v > DB_MAX:
        return DB_MAX
    return v


# A struct used by BufferAnalyzer to emit its results back to AudioLevelMeter
class BufferValues:
    """A struct used by BufferAnalyzer to emit its results back to AudioLevelMeter."""
    def __init__(self, nChannels):
        self.peaks = [0.0] * nChannels
        self.squares = [0.0] * nChannels


class BufferAnalyzer(QObject):
    """A worker class analyzing incoming buffers on a separate worker thread."""
    valuesReady = Signal(BufferValues)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.m_stopRequested = False

    def requestStop(self):
        self.m_stopRequested = True

    @Slot(QAudioBuffer, int)
    def analyzeBuffer(self, buffer, maxChannelsToAnalyze):
        """Analyzes an audio buffer and emits its peak and sumOfSquares values.
           Skips remaining frames if self.m_stopRequested is set to true."""

        if QThread.currentThread().isInterruptionRequested():
            return  # Interrupted by ~AudioLevelMeter, skipping remaining buffers in signal queue

        self.m_stopRequested = False

        channelCount = buffer.format().channelCount()
        channelsToAnalyze = min(channelCount, maxChannelsToAnalyze)

        values = BufferValues(channelsToAnalyze)

        bufferData = buffer.constData()
        bufferSize = len(bufferData)
        bytesPerSample = buffer.format().bytesPerSample()

        for i in range(0, bufferSize, bytesPerSample * channelCount):
            if self.m_stopRequested:
                framesSkipped = (bufferSize - i) / channelCount
                print("BufferAnalyzer::analyzeBuffer skipped", framesSkipped, "out of",
                      buffer.frameCount(), "frames")
                # Emit incomplete values also when stop is requested to get some audio level readout
                # even if frames are being skipped for every buffer. Displayed levels will be
                # inaccurate.
                break

            for channelIndex in range(0, channelsToAnalyze):
                offset = i + bytesPerSample * channelIndex
                sample = buffer.format().normalizedSampleValue(bufferData[offset:])
                values.peaks[channelIndex] = max(values.peaks[channelIndex], abs(sample))
                values.squares[channelIndex] += sample * sample

        self.valuesReady.emit(values)


class MeterChannel(QWidget):
    """A custom QWidget representing an audio channel in the audio level meter. It serves
       both as a model for the channels's peak and RMS values and as a view using the overridden
       paintEvent()."""
    def __init__(self, parent):
        super().__init__(parent)
        self.m_peakDecayRate = 0.0
        self.m_rmsDecayRate = 0.0
        self.m_peak = DB_MIN
        self.m_rms = DB_MIN
        self.m_sumOfSquares = 0.0
        self.m_sumOfSquaresQueue = []
        self.m_peakBrush = QBrush(PEAK_COLOR)
        self.m_rmsBrush = QBrush(RMS_COLOR)

    def normalize(self, dB):
        """# Normalizes a dB value for visualization."""
        return (dB - DB_MIN) / (DB_MAX - DB_MIN)

    def clearRmsData(self):
        """Clears the data used to calculate RMS values."""
        self.m_sumOfSquares = 0.0
        self.m_sumOfSquaresQueue = []

    def decayPeak(self):
        """Decays self.m_peak value by DB_DECAY_PER_UPDATE with ease-in animation based
           on DECAY_EASE_IN_TIME."""
        peak = self.m_peak
        if qFuzzyCompare(peak, DB_MIN):
            return

        cubicEaseInFactor = self.m_peakDecayRate * self.m_peakDecayRate * self.m_peakDecayRate
        self.m_peak = max(DB_MIN, peak - DB_DECAY_PER_UPDATE * cubicEaseInFactor)

        if self.m_peakDecayRate < 1:
            self.m_peakDecayRate += float(UPDATE_INTERVAL) / float(DECAY_EASE_IN_TIME)
            if self.m_peakDecayRate > 1.0:
                self.m_peakDecayRate = 1.0

    def decayRms(self):
        """Decays self.m_rms value by DB_DECAY_PER_UPDATE with ease-in animation based on
           DECAY_EASE_IN_TIME."""
        rms = self.m_rms
        if qFuzzyCompare(rms, DB_MIN):
            return

        cubicEaseInFactor = self.m_rmsDecayRate * self.m_rmsDecayRate * self.m_rmsDecayRate
        self.m_rms = max(DB_MIN, rms - DB_DECAY_PER_UPDATE * cubicEaseInFactor)

        if self.m_rmsDecayRate < 1:
            self.m_rmsDecayRate += float(UPDATE_INTERVAL) / float(DECAY_EASE_IN_TIME)
            if self.m_rmsDecayRate > 1.0:
                self.m_rmsDecayRate = 1.0

    def updatePeak(self, sampleValue):
        """Updates self.m_peak and resets self.m_peakDecayRate if sampleValue > self.m_peak."""
        dB = amplitudeToDb(sampleValue)
        if dB > self.m_peak:
            self.m_peakDecayRate = 0
            self.m_peak = dB

    def updateRms(self, sumOfSquaresForOneBuffer, duration, frameCount):
        """Calculates current RMS. Resets self.m_rmsDecayRate and updates self.m_rms
           if current RMS > self.m_rms."""

        # Add the new sumOfSquares to the queue and update the total
        self.m_sumOfSquaresQueue.append(sumOfSquaresForOneBuffer)
        self.m_sumOfSquares += sumOfSquaresForOneBuffer

        # Remove the oldest sumOfSquares to stay within the RMS window
        if len(self.m_sumOfSquaresQueue) * duration > RMS_WINDOW:
            self.m_sumOfSquares -= self.m_sumOfSquaresQueue[0]
            del self.m_sumOfSquaresQueue[0]

        # Fix negative values caused by floating point precision errors
        if self.m_sumOfSquares < 0:
            self.m_sumOfSquares = 0

        # Calculate the new RMS value
        if self.m_sumOfSquares > 0 and self.m_sumOfSquaresQueue:
            newRms = sqrt(self.m_sumOfSquares / (frameCount * len(self.m_sumOfSquaresQueue)))
            dB = amplitudeToDb(newRms)
            if dB > self.m_rms:
                self.m_rmsDecayRate = 0
                self.m_rms = dB

    def paintEvent(self, event):
        """Paints the level bar of the meter channel based on the decayed peak and rms values."""
        if qFuzzyCompare(self.m_peak, DB_MIN) and qFuzzyCompare(self.m_rms, DB_MIN):
            return  # Nothing to paint

        peakLevel = self.normalize(self.m_peak)
        rmsLevel = self.normalize(self.m_rms)

        with QPainter(self) as painter:
            rect = QRectF(0, self.height(), self.width(), -peakLevel * self.height())
            painter.fillRect(rect, self.m_peakBrush)  # Paint the peak level
            rect.setHeight(-rmsLevel * self.height())
            painter.fillRect(rect, self.m_rmsBrush)  # Paint the RMS level


class AudioLevelMeter(QWidget):
    """The audio level meter´s parent widget class. It acts as a controller
       for the MeterChannel widgets and the BufferAnalyzer worker."""

    newBuffer = Signal(QAudioBuffer, int)

    def __init__(self, parent=None):
        super().__init__(parent)
        self.m_isOn = True
        self.m_isActive = False
        self.m_channels = []
        self.m_channelCount = 0
        self.m_bufferDurationMs = 0
        self.m_frameCount = 0
        self.m_highestPeak = 0.0
        self.m_updateTimer = QTimer()
        self.m_deactivationTimer = QTimer()
        self.m_peakLabelHoldTimer = QTimer()
        self.m_peakLabel = None
        self.m_onOffButton = None
        self.m_bufferAnalyzer = None
        self.m_analyzerThread = QThread()

        # Layout and background color
        self.setSizePolicy(QSizePolicy.Policy.Fixed, QSizePolicy.Policy.Preferred)
        self.setMinimumWidth(WIDGET_WIDTH)
        currentPalette = self.palette()
        currentPalette.setColor(QPalette.ColorRole.Window,
                                currentPalette.color(QPalette.ColorRole.Base))
        self.setPalette(currentPalette)
        self.setAutoFillBackground(True)
        mainLayout = QVBoxLayout(self)
        mainLayout.setSpacing(2)
        mainLayout.setContentsMargins(0, 0, 0, 0)

        # Meter channels
        meterChannelLayout = QHBoxLayout()
        meterChannelLayout.setContentsMargins(2, 2, 2, 2)
        meterChannelLayout.setSpacing(2)
        for i in range(0, MAX_CHANNELS):
            channel = MeterChannel(self)
            meterChannelLayout.addWidget(channel)
            self.m_channels.append(channel)
        mainLayout.addLayout(meterChannelLayout)

        # Peak label
        self.m_peakLabel = QLabel("-", self)
        self.m_peakLabel.setAlignment(Qt.AlignmentFlag.AlignCenter)
        font = QApplication.font()
        font.setPointSize(10)
        self.m_peakLabel.setFont(font)
        mainLayout.addWidget(self.m_peakLabel)
        mainLayout.setStretch(0, 1)

        # On/off button
        self.m_onOffButton = QToolButton(self)
        mainLayout.addWidget(self.m_onOffButton)
        self.m_onOffButton.setMaximumWidth(WIDGET_WIDTH)
        self.m_onOffButton.setText("On")
        self.m_onOffButton.setCheckable(True)
        self.m_onOffButton.setChecked(True)
        self.m_onOffButton.clicked.connect(self.toggleOnOff)

        # Timer triggering update of the audio level bars
        self.m_updateTimer.timeout.connect(self.updateBars)

        # Timer postponing deactivation of update timer to allow meters to fade to 0
        self.m_deactivationTimer.timeout.connect(self.m_updateTimer.stop)
        self.m_deactivationTimer.setSingleShot(True)

        # Timer resetting the peak label
        self.m_peakLabelHoldTimer.timeout.connect(self.resetPeakLabel)
        self.m_peakLabelHoldTimer.setSingleShot(True)

        # Buffer analyzer and worker thread that analyzes incoming buffers
        self.m_bufferAnalyzer = BufferAnalyzer()
        self.m_bufferAnalyzer.moveToThread(self.m_analyzerThread)
        self.m_analyzerThread.finished.connect(self.m_bufferAnalyzer.deleteLater)
        self.newBuffer.connect(self.m_bufferAnalyzer.analyzeBuffer)
        self.m_bufferAnalyzer.valuesReady.connect(self.updateValues)
        self.m_analyzerThread.start()

    def closeRequest(self):
        self.m_analyzerThread.requestInterruption()
        self.m_bufferAnalyzer.requestStop()
        self.m_analyzerThread.quit()
        self.m_analyzerThread.wait()

    @Slot(QAudioBuffer)
    def onAudioBufferReceived(self, buffer):
        """Receives a buffer from QAudioBufferOutput and triggers BufferAnalyzer to analyze it."""
        if not self.m_isOn or not buffer.isValid() or not buffer.format().isValid():
            return

        if not self.m_isActive:
            self.activate()

        # Update internal values to match the current audio stream
        self.updateChannelCount(buffer.format().channelCount())
        self.m_frameCount = buffer.frameCount()
        self.m_bufferDurationMs = buffer.duration() / 1000

        # Stop any ongoing analysis, skipping remaining frames
        self.m_bufferAnalyzer.requestStop()

        self.newBuffer.emit(buffer, self.m_channelCount)

    @Slot(BufferValues)
    def updateValues(self, values):
        """Updates peak/RMS values and peak label."""
        if not self.m_isActive:
            return  # Discard incoming values from BufferAnalyzer

        bufferPeak = 0.0
        for i in range(0, len(values.peaks)):
            bufferPeak = max(bufferPeak, values.peaks[i])
            self.m_channels[i].updatePeak(values.peaks[i])
            self.m_channels[i].updateRms(values.squares[i], self.m_bufferDurationMs,
                                         self.m_frameCount)
        self.updatePeakLabel(bufferPeak)

    def updatePeakLabel(self, peak):
        """Updates peak label and restarts self.m_peakLabelHoldTimer
           if peak >= self.m_highestPeak."""
        if peak < self.m_highestPeak:
            return

        self.m_peakLabelHoldTimer.start(PEAK_LABEL_HOLD_TIME)

        if qFuzzyCompare(peak, self.m_highestPeak):
            return

        self.m_highestPeak = peak
        dB = amplitudeToDb(self.m_highestPeak)
        self.m_peakLabel.setText(f"{int(dB)}")

    @Slot()
    def resetPeakLabel(self):
        """Resets peak label. Called when self.m_labelHoldTimer timeouts."""
        self.m_highestPeak = 0.0
        self.m_peakLabel.setText(f"{DB_MIN}" if self.m_isOn else "")

    def clearAllRmsData(self):
        """Clears internal data used to calculate RMS values."""
        for channel in self.m_channels.copy():
            channel.clearRmsData()

    @Slot()
    def activate(self):
        """Starts the update timer that updates the meter bar."""
        self.m_isActive = True
        self.m_deactivationTimer.stop()
        self.m_updateTimer.start(UPDATE_INTERVAL)

    @Slot()
    def deactivate(self):
        """Start the deactiviation timer that eventually stops the update timer."""
        self.m_isActive = False
        self.clearAllRmsData()
        # Calculate the time it takes to decay fram max to min dB
        interval = (DB_MAX - DB_MIN) / (DB_DECAY_PER_SECOND / 1000) + DECAY_EASE_IN_TIME
        self.m_deactivationTimer.start(interval)

    @Slot()
    def updateBars(self):
        """Decays internal peak and RMS values and triggers repainting of meter bars."""
        for i in range(0, self.m_channelCount):
            channel = self.m_channels[i]
            channel.decayPeak()
            channel.decayRms()
            channel.update()  # Trigger paint event

    @Slot()
    def toggleOnOff(self):
        """Toggles between on (activated) and off (deactivated) state."""
        self.m_isOn = not self.m_isOn
        if not self.m_isOn:
            self.deactivate()
        else:
            self.activate()
        self.m_onOffButton.setText("On" if self.m_isOn else "Off")

    def updateChannelCount(self, channelCount):
        """Updates the number of visible MeterChannel widgets."""
        if (channelCount == self.m_channelCount
                or (channelCount > MAX_CHANNELS and MAX_CHANNELS == self.m_channelCount)):
            return

        self.m_channelCount = min(channelCount, MAX_CHANNELS)
        for i in range(0, MAX_CHANNELS):
            self.m_channels[i].setVisible(i < self.m_channelCount)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the qml/tutorials/extending-qml/chapter1-basics example from Qt v5.x"""

from pathlib import Path
import sys

from PySide6.QtCore import Property, Signal
from PySide6.QtGui import QGuiApplication, QPen, QPainter, QColor
from PySide6.QtQml import QmlElement
from PySide6.QtQuick import QQuickPaintedItem, QQuickView

# To be used on the @QmlElement decorator
# (QML_IMPORT_MINOR_VERSION is optional)
QML_IMPORT_NAME = "Charts"
QML_IMPORT_MAJOR_VERSION = 1


@QmlElement
class PieChart (QQuickPaintedItem):

    nameChanged = Signal()

    def __init__(self, parent=None):
        QQuickPaintedItem.__init__(self, parent)
        self._name = u''
        self._color = QColor()

    def paint(self, painter):
        pen = QPen(self.color, 2)
        painter.setPen(pen)
        painter.setRenderHints(QPainter.RenderHint.Antialiasing, True)
        painter.drawPie(self.boundingRect().adjusted(1, 1, -1, -1), 90 * 16, 290 * 16)

    @Property(QColor, final=True)
    def color(self):
        return self._color

    @color.setter
    def color(self, value):
        self._color = value

    @Property(str, notify=nameChanged, final=True)
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value


if __name__ == '__main__':
    app = QGuiApplication(sys.argv)

    view = QQuickView()
    view.setResizeMode(QQuickView.ResizeMode.SizeRootObjectToView)
    view.engine().addImportPath(Path(__file__).parent)
    view.loadFromModule("Charts", "App")
    if view.status() == QQuickView.Status.Error:
        sys.exit(-1)
    view.show()
    res = app.exec()
    # Deleting the view before it goes out of scope is required to make sure all child QML instances
    # are destroyed in the correct order.
    del view
    sys.exit(res)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()
from smart import Integer, Integer2, StdUniquePtrTestBench, StdUniquePtrVirtualMethodTester, std


def call_func_on_ptr(ptr):
    ptr.printInteger()


class VirtualTester(StdUniquePtrVirtualMethodTester):

    def doCreateInteger(self, v):
        iv = Integer()  # Construct from pointee
        iv.setValue(2 * v)
        return std.unique_ptr_Integer(iv)

    def doModifyIntegerByRef(self, p):
        return 2 * p.value()

    def doModifyIntegerByValue(self, p):
        return 2 * p.value()


class StdUniquePtrTests(unittest.TestCase):
    def testInteger(self):
        p = StdUniquePtrTestBench.createInteger()
        StdUniquePtrTestBench.printInteger(p)  # unique_ptr by ref
        self.assertTrue(p)

        call_func_on_ptr(p)
        self.assertTrue(p)

        StdUniquePtrTestBench.takeInteger(p)  # unique_ptr by value, takes pointee
        self.assertFalse(p)

        np = StdUniquePtrTestBench.createNullInteger()
        StdUniquePtrTestBench.printInteger(np)
        self.assertFalse(np)
        self.assertRaises(AttributeError, call_func_on_ptr, np)

        iv = Integer()  # Construct from pointee
        iv.setValue(42)
        np = std.unique_ptr_Integer(iv)
        self.assertEqual(np.value(), 42)

    def test_derived(self):
        iv2 = Integer2()  # Construct from pointee
        iv2.setValue(42)
        p = std.unique_ptr_Smart_Integer2(iv2)
        self.assertEqual(p.value(), 42)
        StdUniquePtrTestBench.printInteger2(p)  # unique_ptr by ref
        self.assertTrue(p)
        StdUniquePtrTestBench.printInteger(p)  # conversion
        # FIXME: This fails, pointer is moved in value conversion
        # self.assertTrue(p)

    def testInt(self):
        p = StdUniquePtrTestBench.createInt()  # unique_ptr by ref
        StdUniquePtrTestBench.printInt(p)
        StdUniquePtrTestBench.takeInt(p)  # unique_ptr by value, takes pointee
        self.assertFalse(p)

        np = StdUniquePtrTestBench.createNullInt()
        StdUniquePtrTestBench.printInt(np)
        self.assertFalse(np)

    def testVirtuals(self):
        """Test whether code generating virtual function overrides is generated
           correctly."""
        p = StdUniquePtrTestBench.createInteger()
        p.setValue(42)
        v = StdUniquePtrVirtualMethodTester()
        self.assertTrue(v.testCreateInteger(42, 42))
        self.assertTrue(v.testModifyIntegerByRef(42, 43))  # Default implementation increments
        self.assertTrue(v.testModifyIntegerValue(42, 43))

        v = VirtualTester()  # Reimplemented methods double values
        self.assertTrue(v.testCreateInteger(42, 84))
        self.assertTrue(v.testModifyIntegerByRef(42, 84))
        self.assertTrue(v.testModifyIntegerValue(42, 84))


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QMainWindow
from PySide6.QtUiTools import QUiLoader
from helper.timedqapplication import TimedQApplication


class Gui_Qt(QMainWindow):
    def __init__(self, parent=None):
        super().__init__(parent)

        lLoader = QUiLoader()

        # this used to cause a segfault because the old inject code used to destroy
        # the parent layout
        file = Path(__file__).resolve().parent / 'bug_958.ui'
        assert (file.is_file())
        self._cw = lLoader.load(file, self)

        self.setCentralWidget(self._cw)


class BugTest(TimedQApplication):
    def testCase(self):
        lMain = Gui_Qt()
        lMain.show()
        self.app.exec()


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# Copyright (C) 2013 Riverbank Computing Limited.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Slot, Qt
from PySide6.QtWidgets import (QHBoxLayout, QMainWindow,
                               QMessageBox, QPushButton, QSlider,
                               QVBoxLayout, QWidget)

from glwidget import GLWidget


def _main_window():
    for t in qApp.topLevelWidgets():  # noqa: F821
        if isinstance(t, QMainWindow):
            return t
    return None


class Window(QWidget):
    instances = []  # Keep references when undocked

    def __init__(self, parent=None):
        super().__init__(parent)
        self.instances.append(self)

        self._gl_widget = GLWidget()

        self._x_slider = self.create_slider()
        self._x_slider.valueChanged.connect(self._gl_widget.set_xrotation)
        self._gl_widget.x_rotation_changed.connect(self._x_slider.setValue)

        self._y_slider = self.create_slider()
        self._y_slider.valueChanged.connect(self._gl_widget.set_yrotation)
        self._gl_widget.y_rotation_changed.connect(self._y_slider.setValue)

        self._z_slider = self.create_slider()
        self._z_slider.valueChanged.connect(self._gl_widget.set_zrotation)
        self._gl_widget.z_rotation_changed.connect(self._z_slider.setValue)

        mainLayout = QVBoxLayout(self)
        w = QWidget()
        container = QHBoxLayout(w)
        container.addWidget(self._gl_widget)
        container.addWidget(self._x_slider)
        container.addWidget(self._y_slider)
        container.addWidget(self._z_slider)

        mainLayout.addWidget(w)
        self._dock_btn = QPushButton("Undock")
        self._dock_btn.clicked.connect(self.dock_undock)
        mainLayout.addWidget(self._dock_btn)

        self._x_slider.setValue(15 * 16)
        self._y_slider.setValue(345 * 16)
        self._z_slider.setValue(0 * 16)

        self.setWindowTitle(self.tr("Hello GL"))

    def create_slider(self):
        slider = QSlider(Qt.Orientation.Vertical)

        slider.setRange(0, 360 * 16)
        slider.setSingleStep(16)
        slider.setPageStep(15 * 16)
        slider.setTickInterval(15 * 16)
        slider.setTickPosition(QSlider.TickPosition.TicksRight)
        return slider

    def closeEvent(self, event):
        self.instances.remove(self)
        event.accept()

    def keyPressEvent(self, event):
        if self.isWindow() and event.key() == Qt.Key.Key_Escape:
            self.close()
        else:
            super().keyPressEvent(event)

    @Slot()
    def dock_undock(self):
        if self.parent():
            self.undock()
        else:
            self.dock()

    def dock(self):
        mainWindow = _main_window()
        if not mainWindow or not mainWindow.isVisible():
            QMessageBox.information(self, "Cannot Dock",
                                    "Main window already closed")
            return
        if mainWindow.centralWidget():
            QMessageBox.information(self, "Cannot Dock",
                                    "Main window already occupied")
            return

        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose, False)
        self._dock_btn.setText("Undock")
        mainWindow.setCentralWidget(self)

    def undock(self):
        self.setParent(None)
        self.setAttribute(Qt.WidgetAttribute.WA_DeleteOnClose)
        geometry = self.screen().availableGeometry()
        x = geometry.x() + (geometry.width() - self.width()) / 2
        y = geometry.y() + (geometry.height() - self.height()) / 2
        self.move(x, y)
        self._dock_btn.setText("Dock")
        self.show()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the bluetooth/btscanner example from Qt v6.x"""

import sys

from PySide6.QtWidgets import QApplication

from device import DeviceDiscoveryDialog


if __name__ == '__main__':
    app = QApplication(sys.argv)
    d = DeviceDiscoveryDialog()
    d.exec()
    sys.exit(0)



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QIODevice, QTextStream


class MyIODevice (QIODevice):
    def readData(self, amount):
        return bytes("\0a" * int(amount / 2), "UTF-8")

    def readLineData(self, maxSize):
        return bytes("\0b" * 4, "UTF-8")

    def atEnd(self):
        return False


class TestBug944 (unittest.TestCase):

    def testIt(self):
        device = MyIODevice()
        device.open(QIODevice.OpenModeFlag.ReadOnly)
        s = QTextStream(device)
        self.assertEqual(s.read(4), "\0a\0a")
        self.assertEqual(device.readLine(), "\0b\0b\0b\0b")


if __name__ == "__main__":
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import QObject, QSize, Qt
from PySide6.QtWidgets import (QCheckBox, QComboBox, QCommandLinkButton,
                               QLabel, QHBoxLayout, QSizePolicy,
                               QVBoxLayout, QWidget, )
from PySide6.QtDataVisualization import (QAbstract3DSeries, Q3DScatter)

from scatterdatamodifier import ScatterDataModifier


class ScatterGraph(QObject):

    def __init__(self):
        super().__init__()
        self._scatterGraph = Q3DScatter()
        self._container = None
        self._scatterWidget = None

    def initialize(self, minimum_graph_size, maximum_graph_size):
        if not self._scatterGraph.hasContext():
            return -1

        self._scatterWidget = QWidget()
        hLayout = QHBoxLayout(self._scatterWidget)
        self._container = QWidget.createWindowContainer(self._scatterGraph, self._scatterWidget)
        self._container.setMinimumSize(minimum_graph_size)
        self._container.setMaximumSize(maximum_graph_size)
        self._container.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self._container.setFocusPolicy(Qt.StrongFocus)
        hLayout.addWidget(self._container, 1)

        vLayout = QVBoxLayout()
        hLayout.addLayout(vLayout)

        cameraButton = QCommandLinkButton(self._scatterWidget)
        cameraButton.setText("Change camera preset")
        cameraButton.setDescription("Switch between a number of preset camera positions")
        cameraButton.setIconSize(QSize(0, 0))

        itemCountButton = QCommandLinkButton(self._scatterWidget)
        itemCountButton.setText("Toggle item count")
        itemCountButton.setDescription("Switch between 900 and 10000 data points")
        itemCountButton.setIconSize(QSize(0, 0))

        rangeButton = QCommandLinkButton(self._scatterWidget)
        rangeButton.setText("Toggle axis ranges")
        rangeButton.setDescription("Switch between automatic axis ranges and preset ranges")
        rangeButton.setIconSize(QSize(0, 0))

        backgroundCheckBox = QCheckBox(self._scatterWidget)
        backgroundCheckBox.setText("Show background")
        backgroundCheckBox.setChecked(True)

        gridCheckBox = QCheckBox(self._scatterWidget)
        gridCheckBox.setText("Show grid")
        gridCheckBox.setChecked(True)

        smoothCheckBox = QCheckBox(self._scatterWidget)
        smoothCheckBox.setText("Smooth dots")
        smoothCheckBox.setChecked(True)

        itemStyleList = QComboBox(self._scatterWidget)
        itemStyleList.addItem("Sphere", QAbstract3DSeries.MeshSphere)
        itemStyleList.addItem("Cube", QAbstract3DSeries.MeshCube)
        itemStyleList.addItem("Minimal", QAbstract3DSeries.MeshMinimal)
        itemStyleList.addItem("Point", QAbstract3DSeries.MeshPoint)
        itemStyleList.setCurrentIndex(0)

        themeList = QComboBox(self._scatterWidget)
        themeList.addItem("Qt")
        themeList.addItem("Primary Colors")
        themeList.addItem("Digia")
        themeList.addItem("Stone Moss")
        themeList.addItem("Army Blue")
        themeList.addItem("Retro")
        themeList.addItem("Ebony")
        themeList.addItem("Isabelle")
        themeList.setCurrentIndex(3)

        shadowQuality = QComboBox(self._scatterWidget)
        shadowQuality.addItem("None")
        shadowQuality.addItem("Low")
        shadowQuality.addItem("Medium")
        shadowQuality.addItem("High")
        shadowQuality.addItem("Low Soft")
        shadowQuality.addItem("Medium Soft")
        shadowQuality.addItem("High Soft")
        shadowQuality.setCurrentIndex(6)

        vLayout.addWidget(cameraButton)
        vLayout.addWidget(itemCountButton)
        vLayout.addWidget(rangeButton)
        vLayout.addWidget(backgroundCheckBox)
        vLayout.addWidget(gridCheckBox)
        vLayout.addWidget(smoothCheckBox)
        vLayout.addWidget(QLabel("Change dot style"))
        vLayout.addWidget(itemStyleList)
        vLayout.addWidget(QLabel("Change theme"))
        vLayout.addWidget(themeList)
        vLayout.addWidget(QLabel("Adjust shadow quality"))
        vLayout.addWidget(shadowQuality, 1, Qt.AlignTop)

        self._modifier = ScatterDataModifier(self._scatterGraph, self)

        cameraButton.clicked.connect(self._modifier.changePresetCamera)
        itemCountButton.clicked.connect(self._modifier.toggleItemCount)
        rangeButton.clicked.connect(self._modifier.toggleRanges)

        backgroundCheckBox.stateChanged.connect(self._modifier.setBackgroundEnabled)
        gridCheckBox.stateChanged.connect(self._modifier.setGridEnabled)
        smoothCheckBox.stateChanged.connect(self._modifier.setSmoothDots)

        self._modifier.backgroundEnabledChanged.connect(backgroundCheckBox.setChecked)
        self._modifier.gridEnabledChanged.connect(gridCheckBox.setChecked)
        itemStyleList.currentIndexChanged.connect(self._modifier.changeStyle)

        themeList.currentIndexChanged.connect(self._modifier.changeTheme)

        shadowQuality.currentIndexChanged.connect(self._modifier.changeShadowQuality)

        self._modifier.shadowQualityChanged.connect(shadowQuality.setCurrentIndex)
        self._scatterGraph.shadowQualityChanged.connect(self._modifier.shadowQualityUpdatedByVisual)
        return True

    def scatterWidget(self):
        return self._scatterWidget



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for std::map container conversions'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import MapUser


class ExtendedMapUser(MapUser):
    def __init__(self):
        MapUser.__init__(self)
        self.create_map_called = False

    def createMap(self):
        self.create_map_called = True
        return {'two': (complex(2.2, 2.2), 2),
                'three': (complex(3.3, 3.3), 3),
                'five': (complex(5.5, 5.5), 5),
                'seven': (complex(7.7, 7.7), 7)}


class MapConversionTest(unittest.TestCase):
    '''Test case for std::map container conversions'''

    def testReimplementedVirtualMethodCall(self):
        '''Test if a Python override of a virtual method is correctly called from C++.'''
        mu = ExtendedMapUser()
        map_ = mu.callCreateMap()
        self.assertTrue(mu.create_map_called)
        self.assertEqual(type(map_), dict)
        for key, value in map_.items():
            self.assertEqual(type(key), str)
            self.assertEqual(type(value[0]), complex)
            self.assertEqual(type(value[1]), int)

    def testConversionInBothDirections(self):
        '''Test converting a map from Python to C++ and back again.'''
        mu = MapUser()
        map_ = {'odds': [2, 4, 6], 'evens': [3, 5, 7], 'primes': [3, 4, 6]}
        mu.setMap(map_)
        result = mu.getMap()
        self.assertEqual(result, map_)

    def testConversionMapIntKeyValueTypeValue(self):
        '''C++ signature: MapUser::passMapIntValueType(const std::map<int, const ByteArray>&)'''
        mu = MapUser()
        map_ = {0: 'string'}
        result = mu.passMapIntValueType(map_)
        self.assertEqual(map_, result)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import sys

import numpy as np
from scipy.stats import norm
from matplotlib.figure import Figure
from matplotlib.backends.backend_qtagg import FigureCanvas
from matplotlib.backends.backend_qtagg import NavigationToolbar2QT
from PySide6.QtCore import Slot
from PySide6.QtWidgets import (
    QApplication,
    QWidget,
    QDoubleSpinBox,
    QVBoxLayout,
    QHBoxLayout,
)


"""This example implements the interaction between Qt Widgets and a 2D
matplotlib plot showing a gaussian curve with scipy"""


class PlotWidget(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)

        #  create widgets
        self.view = FigureCanvas(Figure(figsize=(5, 3)))
        self.axes = self.view.figure.subplots()
        self.toolbar = NavigationToolbar2QT(self.view, self)
        self.mu_input = QDoubleSpinBox()
        self.std_input = QDoubleSpinBox()
        self.mu_input.setPrefix("μ: ")
        self.std_input.setPrefix("σ: ")
        self.std_input.setValue(10)

        #  Create layout
        input_layout = QHBoxLayout()
        input_layout.addWidget(self.mu_input)
        input_layout.addWidget(self.std_input)
        vlayout = QVBoxLayout()
        vlayout.addWidget(self.toolbar)
        vlayout.addWidget(self.view)
        vlayout.addLayout(input_layout)
        self.setLayout(vlayout)

        # connect inputs with on_change method
        self.mu_input.valueChanged.connect(self.on_change)
        self.std_input.valueChanged.connect(self.on_change)

        self.on_change()

    @Slot()
    def on_change(self):
        """ Update the plot with the current input values """
        mu = self.mu_input.value()
        std = self.std_input.value()

        x = np.linspace(-100, 100)
        y = norm.pdf(x, mu, std)

        self.axes.clear()
        self.axes.plot(x, y)
        self.view.draw()


if __name__ == "__main__":

    app = QApplication(sys.argv)
    w = PlotWidget()
    w.show()
    sys.exit(app.exec())



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

import json

from PySide6.QtWidgets import (QLabel, QLineEdit, QListWidget,
                               QListWidgetItem, QMenu, QTreeView)
from PySide6.QtGui import (QAction, QIcon, QKeySequence,
                           QPixmap, QTextDocument)
from PySide6.QtCore import (QAbstractItemModel, QDir,
                            QIODevice, QModelIndex,
                            QPoint, QSize, Qt, Slot)

from abstractviewer import AbstractViewer


def resizeToContents(tree):
    for i in range(0, tree.header().count()):
        tree.resizeColumnToContents(i)


class JsonTreeItem:

    def __init__(self, parent=None):
        self._key = ""
        self._value = None
        self._children = []
        self._parent = parent

    def key(self):
        return self._key

    def value(self):
        return self._value

    def appendChild(self, item):
        self._children.append(item)

    def child(self, row):
        return self._children[row]

    def parent(self):
        return self._parent

    def childCount(self):
        return len(self._children)

    def row(self):
        if self._parent:
            return self._parent._children.index(self)
        return 0

    def setKey(self, key):
        self._key = key

    def setValue(self, value):
        self._value = value

    @staticmethod
    def load(value, parent=None):
        rootItem = JsonTreeItem(parent)
        rootItem.setKey("root")

        if isinstance(value, dict):
            for key, val in value.items():
                child = JsonTreeItem.load(val, rootItem)
                child.setKey(key)
                rootItem.appendChild(child)

        elif isinstance(value, list):
            for index, val in enumerate(value):
                child = JsonTreeItem.load(val, rootItem)
                child.setKey(f"{index}")
                rootItem.appendChild(child)

        else:
            rootItem.setValue(value)

        return rootItem


class JsonItemModel(QAbstractItemModel):

    def columnCount(self, index=QModelIndex()):
        return 2

    def itemFromIndex(self, index):
        return index.internalPointer()

    def __init__(self, doc, parent):
        super().__init__(parent)
        self._textItem = JsonTreeItem()

        # Append header lines
        self._headers = ["Key", "Value"]

        # Reset the model. Root can either be a value or an array.
        self.beginResetModel()
        self._textItem = JsonTreeItem.load(doc) if doc else JsonTreeItem()
        self.endResetModel()

    def data(self, index, role):
        if not index.isValid():
            return None

        item = self.itemFromIndex(index)
        if role == Qt.ItemDataRole.DisplayRole:
            if index.column() == 0:
                return item.key()
            if index.column() == 1:
                return item.value()
        elif role == Qt.ItemDataRole.EditRole:
            if index.column() == 1:
                return item.value()
        return None

    def headerData(self, section, orientation, role):
        return (self._headers[section]
                if role == Qt.ItemDataRole.DisplayRole and orientation == Qt.Orientation.Horizontal
                else None)

    def index(self, row, column, parent=QModelIndex()):
        if not self.hasIndex(row, column, parent):
            return None

        parentItem = JsonTreeItem()

        if not parent.isValid():
            parentItem = self._textItem
        else:
            parentItem = self.itemFromIndex(parent)

        childItem = parentItem.child(row)
        if childItem:
            return self.createIndex(row, column, childItem)
        return None

    def parent(self, index):
        if not index.isValid():
            return None

        childItem = self.itemFromIndex(index)
        parentItem = childItem.parent()

        if parentItem == self._textItem:
            return QModelIndex()

        return self.createIndex(parentItem.row(), 0, parentItem)

    def rowCount(self, parent=QModelIndex()):
        parentItem = JsonTreeItem()
        if parent.column() > 0:
            return 0

        if not parent.isValid():
            parentItem = self._textItem
        else:
            parentItem = self.itemFromIndex(parent)
        return parentItem.childCount()


class JsonViewer(AbstractViewer):

    def __init__(self):
        super().__init__()
        self._tree = QTreeView()
        self._toplevel = None
        self._text = ""
        self._searchKey = None
        self.uiInitialized.connect(self.setupJsonUi)

    def init(self, file, parent, mainWindow):
        self._tree = QTreeView(parent)
        super().init(file, self._tree, mainWindow)

    def viewerName(self):
        return "JsonViewer"

    def supportedMimeTypes(self):
        return ["application/json"]

    @Slot()
    def setupJsonUi(self):
        # Build Menus and toolbars
        menu = self.addMenu("Json")
        tb = self.addToolBar("Json Actions")

        zoomInIcon = QIcon.fromTheme(QIcon.ThemeIcon.ZoomIn)
        a = menu.addAction(zoomInIcon, "&+Expand all", self._tree.expandAll)
        tb.addAction(a)
        a.setPriority(QAction.Priority.LowPriority)
        a.setShortcut(QKeySequence.StandardKey.New)

        zoomOutIcon = QIcon.fromTheme(QIcon.ThemeIcon.ZoomOut)
        a = menu.addAction(zoomOutIcon, "&-Collapse all", self._tree.collapseAll)
        tb.addAction(a)
        a.setPriority(QAction.Priority.LowPriority)
        a.setShortcut(QKeySequence.StandardKey.New)

        if not self._searchKey:
            self._searchKey = QLineEdit(tb)

        label = QLabel(tb)
        magnifier = QPixmap(":/icons/images/magnifier.png").scaled(QSize(28, 28))
        label.setPixmap(magnifier)
        tb.addWidget(label)
        tb.addWidget(self._searchKey)
        self._searchKey.textEdited.connect(self._tree.keyboardSearch)

        if not self.openJsonFile():
            return

        # Populate bookmarks with toplevel
        self._uiAssets_tabs.clear()
        self._toplevel = QListWidget(self._uiAssets_tabs)
        self._uiAssets_tabs.addTab(self._toplevel, "Bookmarks")
        for i in range(0, self._tree.model().rowCount()):
            index = self._tree.model().index(i, 0)
            self._toplevel.addItem(index.data())
            item = self._toplevel.item(i)
            item.setData(Qt.ItemDataRole.UserRole, index)
            item.setToolTip(f"Toplevel Item {i}")

        self._toplevel.setAcceptDrops(True)
        self._tree.setDragEnabled(True)
        self._tree.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)
        self._toplevel.setContextMenuPolicy(Qt.ContextMenuPolicy.CustomContextMenu)

        self._toplevel.itemClicked.connect(self.onTopLevelItemClicked)
        self._toplevel.itemDoubleClicked.connect(self.onTopLevelItemDoubleClicked)
        self._toplevel.customContextMenuRequested.connect(self.onBookmarkMenuRequested)
        self._tree.customContextMenuRequested.connect(self.onJsonMenuRequested)

        # Connect back and forward
        self._uiAssets_back.triggered.connect(self._back)
        self._uiAssets_forward.triggered.connect(self._forward)

    @Slot()
    def _back(self):
        index = self._tree.indexAbove(self._tree.currentIndex())
        if index.isValid():
            self._tree.setCurrentIndex(index)

    @Slot()
    def _forward(self):
        current = self._tree.currentIndex()
        next = self._tree.indexBelow(current)
        if next.isValid():
            self._tree.setCurrentIndex(next)
            return
        # Expand last item to go beyond
        if not self._tree.isExpanded(current):
            self._tree.expand(current)
            next = self._tree.indexBelow(current)
            if next.isValid():
                self._tree.setCurrentIndex(next)

    def openJsonFile(self):
        self.disablePrinting()
        file_name = QDir.toNativeSeparators(self._file.fileName())
        type = "open"
        self._file.open(QIODevice.OpenModeFlag.ReadOnly)
        self._text = self._file.readAll().data().decode("utf-8")
        self._file.close()

        data = None
        message = None
        try:
            data = json.loads(self._text)
            message = f"Json document {file_name} opened"
            model = JsonItemModel(data, self)
            self._tree.setModel(model)
        except ValueError as e:
            message = f"Unable to parse Json document from {file_name}: {e}"
        self.statusMessage(message, type)
        self.maybeEnablePrinting()

        return self._tree.model() is not None

    def indexOf(self, item):
        return QModelIndex(item.data(Qt.ItemDataRole.UserRole))

    @Slot(QListWidgetItem)
    def onTopLevelItemClicked(self, item):
        """Move to the clicked toplevel index"""
        # return in the unlikely case that the tree has not been built
        if not self._tree.model():
            return

        index = self.indexOf(item)
        if not index.isValid():
            return

        self._tree.setCurrentIndex(index)

    @Slot(QListWidgetItem)
    def onTopLevelItemDoubleClicked(self, item):
        """Toggle double clicked index between collaps/expand"""

        # return in the unlikely case that the tree has not been built
        if not self._tree.model():
            return

        index = self.indexOf(item)
        if not index.isValid():
            return

        if self._tree.isExpanded(index):
            self._tree.collapse(index)
            return

        # Make sure the node and all parents are expanded
        while index.isValid():
            self._tree.expand(index)
            index = index.parent()

    @Slot(QPoint)
    def onJsonMenuRequested(self, pos):
        index = self._tree.indexAt(pos)
        if not index.isValid():
            return

        # Don't show a context menu, if the index is already a bookmark
        for i in range(0, self._toplevel.count()):
            if self.indexOf(self._toplevel.item(i)) == index:
                return

        menu = QMenu(self._tree)
        action = QAction("Add bookmark")
        action.setData(index)
        menu.addAction(action)
        action.triggered.connect(self.onBookmarkAdded)
        menu.exec(self._tree.mapToGlobal(pos))

    @Slot(QPoint)
    def onBookmarkMenuRequested(self, pos):
        item = self._toplevel.itemAt(pos)
        if not item:
            return

        # Don't delete toplevel items
        index = self.indexOf(item)
        if not index.parent().isValid():
            return

        menu = QMenu()
        action = QAction("Delete bookmark")
        action.setData(self._toplevel.row(item))
        menu.addAction(action)
        action.triggered.connect(self.onBookmarkDeleted)
        menu.exec(self._toplevel.mapToGlobal(pos))

    @Slot()
    def onBookmarkAdded(self):
        action = self.sender()
        if not action:
            return

        index = action.data()
        if not index.isValid():
            return

        item = QListWidgetItem(index.data(Qt.ItemDataRole.DisplayRole), self._toplevel)
        item.setData(Qt.ItemDataRole.UserRole, index)

        # Set a tooltip that shows where the item is located in the tree
        parent = index.parent()
        tooltip = index.data(Qt.ItemDataRole.DisplayRole).toString()
        while parent.isValid():
            tooltip = parent.data(Qt.ItemDataRole.DisplayRole).toString() + "." + tooltip
            parent = parent.parent()

        item.setToolTip(tooltip)

    @Slot()
    def onBookmarkDeleted(self):
        action = self.sender()
        if not action:
            return

        row = action.data().toInt()
        if row < 0 or row >= self._toplevel.count():
            return

        self._toplevel.takeItem(row)

    def hasContent(self):
        return bool(self._text)

    def supportsOverview(self):
        return True

    def printDocument(self, printer):
        if not self.hasContent():
            return
        doc = QTextDocument(self._text)
        doc.print_(printer)



class StringListModel(QAbstractListModel):
    def __init__(self, strings, parent=None):
        ...

    def rowCount(self, parent=QModelIndex()):
        ...

    def data(self, index, role):
        ...

    def headerData(self, section, orientation, role=Qt.DisplayRole):
        ...

    def flags(self, index):
        ...

    def setData(self, index, value, role=Qt.EditRole):
        ...

    def insertRows(self, position, rows, parent):
        ...

    def removeRows(self, position, rows, parent):
        ...



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from ui_downloadwidget import Ui_DownloadWidget

from PySide6.QtWebEngineCore import QWebEngineDownloadRequest
from PySide6.QtWidgets import QFrame, QWidget
from PySide6.QtGui import QIcon
from PySide6.QtCore import QElapsedTimer, Signal, Slot


def with_unit(bytes):
    if bytes < (1 << 10):
        return f"{bytes} B"
    if bytes < (1 << 20):
        s = bytes / (1 << 10)
        return f"{int(s)} KiB"
    if bytes < (1 << 30):
        s = bytes / (1 << 20)
        return f"{int(s)} MiB"
    s = bytes / (1 << 30)
    return f"{int(s)} GiB"


class DownloadWidget(QFrame):
    """Displays one ongoing or finished download (QWebEngineDownloadRequest)."""

    # This signal is emitted when the user indicates that they want to remove
    # this download from the downloads list.
    remove_clicked = Signal(QWidget)

    def __init__(self, download, parent=None):
        super().__init__(parent)
        self._download = download
        self._time_added = QElapsedTimer()
        self._time_added.start()
        self._cancel_icon = QIcon.fromTheme(QIcon.ThemeIcon.ProcessStop,
                                            QIcon(":process-stop.png"))
        self._remove_icon = QIcon.fromTheme(QIcon.ThemeIcon.EditClear,
                                            QIcon(":edit-clear.png"))

        self._ui = Ui_DownloadWidget()
        self._ui.setupUi(self)
        self._ui.m_dstName.setText(self._download.downloadFileName())
        self._ui.m_srcUrl.setText(self._download.url().toDisplayString())

        self._ui.m_cancelButton.clicked.connect(self._canceled)

        self._download.totalBytesChanged.connect(self.update_widget)
        self._download.receivedBytesChanged.connect(self.update_widget)

        self._download.stateChanged.connect(self.update_widget)

        self.update_widget()

    @Slot()
    def _canceled(self):
        state = self._download.state()
        if state == QWebEngineDownloadRequest.DownloadInProgress:
            self._download.cancel()
        else:
            self.remove_clicked.emit(self)

    def update_widget(self):
        total_bytes_v = self._download.totalBytes()
        total_bytes = with_unit(total_bytes_v)
        received_bytes_v = self._download.receivedBytes()
        received_bytes = with_unit(received_bytes_v)
        elapsed = self._time_added.elapsed()
        bytes_per_second_v = received_bytes_v / elapsed * 1000 if elapsed else 0
        bytes_per_second = with_unit(bytes_per_second_v)

        state = self._download.state()

        progress_bar = self._ui.m_progressBar
        if state == QWebEngineDownloadRequest.DownloadInProgress:
            if total_bytes_v > 0:
                progress = round(100 * received_bytes_v / total_bytes_v)
                progress_bar.setValue(progress)
                progress_bar.setDisabled(False)
                fmt = f"%p% - {received_bytes} of {total_bytes} downloaded - {bytes_per_second}/s"
                progress_bar.setFormat(fmt)
            else:
                progress_bar.setValue(0)
                progress_bar.setDisabled(False)
                fmt = f"unknown size - {received_bytes} downloaded - {bytes_per_second}/s"
                progress_bar.setFormat(fmt)
        elif state == QWebEngineDownloadRequest.DownloadCompleted:
            progress_bar.setValue(100)
            progress_bar.setDisabled(True)
            fmt = f"completed - {received_bytes} downloaded - {bytes_per_second}/s"
            progress_bar.setFormat(fmt)
        elif state == QWebEngineDownloadRequest.DownloadCancelled:
            progress_bar.setValue(0)
            progress_bar.setDisabled(True)
            fmt = f"cancelled - {received_bytes} downloaded - {bytes_per_second}/s"
            progress_bar.setFormat(fmt)
        elif state == QWebEngineDownloadRequest.DownloadInterrupted:
            progress_bar.setValue(0)
            progress_bar.setDisabled(True)
            fmt = "interrupted: " + self._download.interruptReasonString()
            progress_bar.setFormat(fmt)

        if state == QWebEngineDownloadRequest.DownloadInProgress:
            self._ui.m_cancelButton.setIcon(self._cancel_icon)
            self._ui.m_cancelButton.setToolTip("Stop downloading")
        else:
            self._ui.m_cancelButton.setIcon(self._remove_icon)
            self._ui.m_cancelButton.setToolTip("Remove from list")



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test case for QObject.receivers()'''

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, SIGNAL, SLOT


def cute_slot():
    pass


class TestQObjectReceivers(unittest.TestCase):
    '''Test case for QObject::receivers'''

    def testBasic(self):
        sender = QObject()
        receiver1 = QObject()
        receiver2 = QObject()
        self.assertEqual(sender.receivers(SIGNAL("")), 0)
        sender.destroyed.connect(receiver1.deleteLater)
        old_count = sender.receivers(SIGNAL("destroyed()"))
        sender.destroyed.connect(receiver2.deleteLater)
        self.assertEqual(sender.receivers(SIGNAL("destroyed()")), old_count + 1)
        sender.disconnect(sender, SIGNAL("destroyed()"), receiver2, SLOT("deleteLater()"))
        self.assertEqual(sender.receivers(SIGNAL("destroyed()")), old_count)
        del receiver2
        del receiver1
        del sender
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def testPySlots(self):
        sender = QObject()
        receiver = QObject()
        sender.destroyed.connect(cute_slot)
        old_count = sender.receivers(SIGNAL("destroyed()"))
        sender.destroyed.connect(receiver.deleteLater)
        self.assertEqual(sender.receivers(SIGNAL("destroyed()")), old_count + 1)
        del sender
        del receiver
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def testPySignals(self):
        sender = QObject()
        receiver = QObject()
        sender.connect(sender, SIGNAL("some_dynamic_signal()"), cute_slot)
        self.assertEqual(sender.receivers(SIGNAL("some_dynamic_signal(  )")), 1)
        sender.connect(sender, SIGNAL("some_dynamic_signal()"), receiver, SLOT("deleteLater()"))
        self.assertEqual(sender.receivers(SIGNAL("some_dynamic_signal(  )")), 2)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest
import weakref

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtCore import QObject, Property  # noqa: E402


class MyObject(QObject):
    def __init__(self):
        super().__init__()
        self._value = None

    @Property(int)
    def value(self):
        return self._value

    @value.setter
    # Note: The name of property and setter must be the same, because the
    # object changes its identity all the time. `valueSet` no longer works.
    def value(self, value):
        self._value = value


class PropertyTest(unittest.TestCase):
    def destroyCB(self, obj):
        self._obDestroyed = True

    def testDecorator(self):
        self._obDestroyed = False
        o = MyObject()
        weak = weakref.ref(o, self.destroyCB)  # noqa: F841
        o.value = 10
        self.assertEqual(o._value, 10)
        self.assertEqual(o.value, 10)
        del o
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()
        # PYSIDE-535: Why do I need to do it twice, here?
        gc.collect()
        self.assertTrue(self._obDestroyed)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for QSerialPort'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths  # noqa: E402
init_test_paths(False)

from PySide6.QtSerialPort import QSerialPort, QSerialPortInfo  # noqa: E402
from PySide6.QtCore import QIODevice  # noqa: E402


class QSerialPortTest(unittest.TestCase):
    def testDefaultConstructedPort(self):
        serialPort = QSerialPort()

        self.assertEqual(serialPort.error(), QSerialPort.SerialPortError.NoError)
        self.assertTrue(not serialPort.errorString() == "")

        # properties
        defaultBaudRate = QSerialPort.BaudRate.Baud9600
        self.assertEqual(serialPort.baudRate(), defaultBaudRate)
        self.assertEqual(serialPort.baudRate(QSerialPort.Direction.Input), defaultBaudRate)
        self.assertEqual(serialPort.baudRate(QSerialPort.Direction.Output), defaultBaudRate)
        self.assertEqual(serialPort.dataBits(), QSerialPort.DataBits.Data8)
        self.assertEqual(serialPort.parity(), QSerialPort.Parity.NoParity)
        self.assertEqual(serialPort.stopBits(), QSerialPort.StopBits.OneStop)
        self.assertEqual(serialPort.flowControl(), QSerialPort.FlowControl.NoFlowControl)

        self.assertEqual(serialPort.pinoutSignals(), QSerialPort.PinoutSignal.NoSignal)
        self.assertEqual(serialPort.isRequestToSend(), False)
        self.assertEqual(serialPort.isDataTerminalReady(), False)

        # QIODevice
        self.assertEqual(serialPort.openMode(), QIODevice.OpenModeFlag.NotOpen)
        self.assertTrue(not serialPort.isOpen())
        self.assertTrue(not serialPort.isReadable())
        self.assertTrue(not serialPort.isWritable())
        self.assertTrue(serialPort.isSequential())
        self.assertEqual(serialPort.canReadLine(), False)
        self.assertEqual(serialPort.pos(), 0)
        self.assertEqual(serialPort.size(), 0)
        self.assertTrue(serialPort.atEnd())
        self.assertEqual(serialPort.bytesAvailable(), 0)
        self.assertEqual(serialPort.bytesToWrite(), 0)

    def testOpenExisting(self):
        allportinfos = QSerialPortInfo.availablePorts()
        for portinfo in allportinfos:
            serialPort = QSerialPort(portinfo)
            self.assertEqual(serialPort.portName(), portinfo.portName())


class QSerialPortInfoTest(unittest.TestCase):
    def test_available_ports(self):
        allportinfos = QSerialPortInfo.availablePorts()
        for portinfo in allportinfos:
            portinfo.description()
            portinfo.hasProductIdentifier()
            portinfo.hasVendorIdentifier()
            portinfo.isNull()


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from enum import Enum
from math import sin, cos, degrees

from PySide6.QtCore import Qt
from PySide6.QtDataVisualization import QAbstract3DGraph, Q3DInputHandler


class InputState(Enum):
    StateNormal = 0
    StateDraggingX = 1
    StateDraggingZ = 2
    StateDraggingY = 3


class AxesInputHandler(Q3DInputHandler):

    def __init__(self, graph, parent=None):
        super().__init__(parent)
        self._mousePressed = False
        self._state = InputState.StateNormal
        self._axisX = None
        self._axisZ = None
        self._axisY = None
        self._speedModifier = 15.0

        # Connect to the item selection signal from graph
        graph.selectedElementChanged.connect(self.handleElementSelected)

    def setAxes(self, axisX, axisZ, axisY):
        self._axisX = axisX
        self._axisZ = axisZ
        self._axisY = axisY

    def setDragSpeedModifier(self, modifier):
        self._speedModifier = modifier

    def mousePressEvent(self, event, mousePos):
        super().mousePressEvent(event, mousePos)
        if Qt.LeftButton == event.button():
            self._mousePressed = True

    def mouseMoveEvent(self, event, mousePos):
        # Check if we're trying to drag axis label
        if self._mousePressed and self._state != InputState.StateNormal:
            self.setPreviousInputPos(self.inputPosition())
            self.setInputPosition(mousePos)
            self.handleAxisDragging()
        else:
            super().mouseMoveEvent(event, mousePos)

    def mouseReleaseEvent(self, event, mousePos):
        super().mouseReleaseEvent(event, mousePos)
        self._mousePressed = False
        self._state = InputState.StateNormal

    def handleElementSelected(self, type):
        if type == QAbstract3DGraph.ElementAxisXLabel:
            self._state = InputState.StateDraggingX
        elif type == QAbstract3DGraph.ElementAxisYLabel:
            self._state = InputState.StateDraggingY
        elif type == QAbstract3DGraph.ElementAxisZLabel:
            self._state = InputState.StateDraggingZ
        else:
            self._state = InputState.StateNormal

    def handleAxisDragging(self):
        distance = 0.0
        # Get scene orientation from active camera
        ac = self.scene().activeCamera()
        xRotation = ac.xRotation()
        yRotation = ac.yRotation()

        # Calculate directional drag multipliers based on rotation
        xMulX = cos(degrees(xRotation))
        xMulY = sin(degrees(xRotation))
        zMulX = sin(degrees(xRotation))
        zMulY = cos(degrees(xRotation))

        # Get the drag amount
        move = self.inputPosition() - self.previousInputPos()

        # Flip the effect of y movement if we're viewing from below
        yMove = -move.y() if yRotation < 0 else move.y()

        # Adjust axes
        if self._state == InputState.StateDraggingX:
            distance = (move.x() * xMulX - yMove * xMulY) / self._speedModifier
            self._axisX.setRange(self._axisX.min() - distance,
                                 self._axisX.max() - distance)
        elif self._state == InputState.StateDraggingZ:
            distance = (move.x() * zMulX + yMove * zMulY) / self._speedModifier
            self._axisZ.setRange(self._axisZ.min() + distance,
                                 self._axisZ.max() + distance)
        elif self._state == InputState.StateDraggingY:
            # No need to use adjusted y move here
            distance = move.y() / self._speedModifier
            self._axisY.setRange(self._axisY.min() + distance,
                                 self._axisY.max() + distance)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for libsample's Point multiply operator defined in libother module.'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import Point
from other import Number


class PointOperationsWithNumber(unittest.TestCase):
    '''Test cases for libsample's Point multiply operator defined in libother module.'''

    def testPointTimesInt(self):
        '''sample.Point * int'''
        pt1 = Point(2, 7)
        num = 3
        pt2 = Point(pt1.x() * num, pt1.y() * num)
        self.assertEqual(pt1 * num, pt2)

    def testIntTimesPoint(self):
        '''int * sample.Point'''
        pt1 = Point(2, 7)
        num = 3
        pt2 = Point(pt1.x() * num, pt1.y() * num)
        self.assertEqual(num * pt1, pt2)

    def testPointTimesNumber(self):
        '''sample.Point * other.Number'''
        pt = Point(2, 7)
        num = Number(11)
        self.assertEqual(pt * num.value(), pt * 11)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import gc
import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtCore import QObject, Signal


class BoundAndUnboundSignalsTest(unittest.TestCase):

    def setUp(self):
        self.methods = set(('connect', 'disconnect', 'emit'))

    def tearDown(self):
        del self.methods
        # PYSIDE-535: Need to collect garbage in PyPy to trigger deletion
        gc.collect()

    def testUnboundSignal(self):
        self.assertEqual(type(QObject.destroyed), Signal)
        self.assertFalse(self.methods.issubset(dir(QObject.destroyed)))

    def testBoundSignal(self):
        obj = QObject()
        self.assertNotEqual(type(obj.destroyed), Signal)
        self.assertTrue(self.methods.issubset(dir(obj.destroyed)))


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for overload sorting'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import (CustomOverloadSequence, ImplicitBase, ImplicitConv,
                    ImplicitTarget, SortedOverload)


class Dummy:
    pass


class SimpleOverloadSorting(unittest.TestCase):

    def setUp(self):
        self.obj = SortedOverload()

    def testIntDouble(self):
        '''Overloads with int and double'''
        self.assertEqual(self.obj.overload(3), "int")
        self.assertEqual(self.obj.overload(3.14), "double")

    def testImplicitConvert(self):
        '''Overloads with implicit convertible types'''
        self.assertEqual(self.obj.overload(ImplicitTarget()), "ImplicitTarget")
        self.assertEqual(self.obj.overload(ImplicitBase()), "ImplicitBase")

    def testContainer(self):
        '''Overloads with containers arguments'''
        self.assertEqual(self.obj.overload([ImplicitBase()]), "list(ImplicitBase)")

    def testPyObject(self):
        '''Overloads with PyObject args'''
        self.assertEqual(self.obj.overload(Dummy()), "PyObject")

    def testImplicitOnly(self):
        '''Passing an implicit convertible object to an overload'''
        self.assertTrue(self.obj.implicit_overload(ImplicitTarget()))

    def testPyObjectSort(self):
        self.assertEqual(self.obj.pyObjOverload(1, 2), "int,int")
        self.assertEqual(self.obj.pyObjOverload(object(), 2), "PyObject,int")


class DeepOverloadSorting(unittest.TestCase):

    def setUp(self):
        self.obj = SortedOverload()

    def testPyObject(self):
        '''Deep Overload - (int, PyObject *)'''
        self.assertEqual(self.obj.overloadDeep(1, Dummy()), "PyObject")

    def testImplicit(self):
        '''Deep Overload - (int, ImplicitBase *)'''
        self.assertEqual(self.obj.overloadDeep(1, ImplicitBase()), "ImplicitBase")


class EnumOverIntSorting(unittest.TestCase):
    def testEnumOverInt(self):
        ic = ImplicitConv(ImplicitConv.CtorEnum.CtorTwo)
        self.assertEqual(ic.ctorEnum(), ImplicitConv.CtorEnum.CtorTwo)


class TestCustomOverloadSequence(unittest.TestCase):
    '''Ensure the int-overload (returning v + sizeof(v)) is first as specified via
       overload-number in XML.'''
    def testCustomOverloadSequence(self):
        s = CustomOverloadSequence()
        self.assertEqual(s.overload(42), 46)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QGroupBox

from ui_canbusdeviceinfobox import Ui_CanBusDeviceInfoBox


def _set_readonly_and_compact(box):
    box.setAttribute(Qt.WidgetAttribute.WA_TransparentForMouseEvents)
    box.setFocusPolicy(Qt.FocusPolicy.NoFocus)
    box.setStyleSheet("margin-top:0; margin-bottom:0;")


class CanBusDeviceInfoBox(QGroupBox):

    def __init__(self, parent):
        super().__init__(parent)
        self.m_ui = Ui_CanBusDeviceInfoBox()
        self.m_ui.setupUi(self)
        _set_readonly_and_compact(self.m_ui.isVirtual)
        _set_readonly_and_compact(self.m_ui.isFlexibleDataRateCapable)

    def clear(self):
        self.m_ui.pluginLabel.clear()
        self.m_ui.nameLabel.clear()
        self.m_ui.descriptionLabel.clear()
        self.m_ui.serialNumberLabel.clear()
        self.m_ui.aliasLabel.clear()
        self.m_ui.channelLabel.clear()
        self.m_ui.isVirtual.setChecked(False)
        self.m_ui.isFlexibleDataRateCapable.setChecked(False)

    def set_device_info(self, info):
        self.m_ui.pluginLabel.setText(f"Plugin: {info.plugin()}")
        self.m_ui.nameLabel.setText(f"Name: {info.name()}")
        self.m_ui.descriptionLabel.setText(info.description())
        serial_number = info.serialNumber()
        if not serial_number:
            serial_number = "n/a"
        self.m_ui.serialNumberLabel.setText(f"Serial: {serial_number}")
        alias = info.alias()
        if not alias:
            alias = "n/a"
        self.m_ui.aliasLabel.setText(f"Alias: {alias}")
        self.m_ui.channelLabel.setText(f"Channel: {info.channel()}")
        self.m_ui.isVirtual.setChecked(info.isVirtual())
        self.m_ui.isFlexibleDataRateCapable.setChecked(info.hasFlexibleDataRate())



# Copyright (C) 2023 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the Qt DataVisualization graphgallery example from Qt v6.x"""

import os
import sys

from PySide6.QtCore import QSize
from PySide6.QtWidgets import QApplication, QMessageBox, QTabWidget

from bargraph import BarGraph
from scattergraph import ScatterGraph
from surfacegraph import SurfaceGraph


if __name__ == "__main__":
    os.environ["QSG_RHI_BACKEND"] = "opengl"

    app = QApplication(sys.argv)

    # Create a tab widget for creating own tabs for Q3DBars, Q3DScatter, and Q3DSurface
    tabWidget = QTabWidget()
    tabWidget.setWindowTitle("Graph Gallery")

    screen_size = tabWidget.screen().size()
    minimum_graph_size = QSize(screen_size.width() / 2, screen_size.height() / 1.75)

    # Create bar graph
    bars = BarGraph()
    # Create scatter graph
    scatter = ScatterGraph()
    # Create surface graph
    surface = SurfaceGraph()

    if (not bars.initialize(minimum_graph_size, screen_size)
            or not scatter.initialize(minimum_graph_size, screen_size)
            or not surface.initialize(minimum_graph_size, screen_size)):
        QMessageBox.warning(None, "Graph Gallery", "Couldn't initialize the OpenGL context.")
        sys.exit(-1)

    # Add bars widget
    tabWidget.addTab(bars.barsWidget(), "Bar Graph")
    # Add scatter widget
    tabWidget.addTab(scatter.scatterWidget(), "Scatter Graph")
    # Add surface widget
    tabWidget.addTab(surface.surfaceWidget(), "Surface Graph")

    tabWidget.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

"""PySide6 port of the Model Data example from Qt v5.x"""

import sys
from random import randrange

from PySide6.QtCore import QAbstractTableModel, QModelIndex, QRect, Qt
from PySide6.QtGui import QColor, QPainter
from PySide6.QtWidgets import (QApplication, QGridLayout, QHeaderView,
                               QTableView, QWidget)
from PySide6.QtCharts import QChart, QChartView, QLineSeries, QVXYModelMapper


class CustomTableModel(QAbstractTableModel):
    def __init__(self):
        super().__init__()
        self.input_data = []
        self.mapping = {}
        self.column_count = 4
        self.row_count = 15

        for i in range(self.row_count):
            data_vec = [0] * self.column_count
            for k in range(len(data_vec)):
                if k % 2 == 0:
                    data_vec[k] = i * 50 + randrange(30)
                else:
                    data_vec[k] = randrange(100)
            self.input_data.append(data_vec)

    def rowCount(self, parent=QModelIndex()):
        return len(self.input_data)

    def columnCount(self, parent=QModelIndex()):
        return self.column_count

    def headerData(self, section, orientation, role):
        if role != Qt.ItemDataRole.DisplayRole:
            return None

        if orientation == Qt.Orientation.Horizontal:
            if section % 2 == 0:
                return "x"
            else:
                return "y"
        else:
            return str(section + 1)

    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
        if role == Qt.ItemDataRole.DisplayRole:
            return self.input_data[index.row()][index.column()]
        elif role == Qt.ItemDataRole.EditRole:
            return self.input_data[index.row()][index.column()]
        elif role == Qt.ItemDataRole.BackgroundRole:
            for color, rect in self.mapping.items():
                if rect.contains(index.column(), index.row()):
                    return QColor(color)
            # cell not mapped return white color
            return QColor(Qt.white)
        return None

    def setData(self, index, value, role=Qt.ItemDataRole.EditRole):
        if index.isValid() and role == Qt.ItemDataRole.EditRole:
            self.input_data[index.row()][index.column()] = float(value)
            self.dataChanged.emit(index, index)
            return True
        return False

    def flags(self, index):
        return Qt.ItemIsEnabled | Qt.ItemIsEditable | Qt.ItemIsSelectable

    def add_mapping(self, color, area):
        self.mapping[color] = area

    def clear_mapping(self):
        self.mapping = {}


class TableWidget(QWidget):
    def __init__(self):
        super().__init__()

        self.model = CustomTableModel()

        self.table_view = QTableView()
        self.table_view.setModel(self.model)
        self.table_view.horizontalHeader().setSectionResizeMode(QHeaderView.Stretch)
        self.table_view.verticalHeader().setSectionResizeMode(QHeaderView.Stretch)

        self.chart = QChart()
        self.chart.setAnimationOptions(QChart.AllAnimations)

        self.series = QLineSeries()
        self.series.setName("Line 1")
        self.mapper = QVXYModelMapper(self)
        self.mapper.setXColumn(0)
        self.mapper.setYColumn(1)
        self.mapper.setSeries(self.series)
        self.mapper.setModel(self.model)
        self.chart.addSeries(self.series)

        # get the color of the series and use it for showing the mapped area
        self.model.add_mapping(self.series.pen().color().name(),
                               QRect(0, 0, 2, self.model.rowCount()))

        # series 2
        self.series = QLineSeries()
        self.series.setName("Line 2")

        self.mapper = QVXYModelMapper(self)
        self.mapper.setXColumn(2)
        self.mapper.setYColumn(3)
        self.mapper.setSeries(self.series)
        self.mapper.setModel(self.model)
        self.chart.addSeries(self.series)

        # get the color of the series and use it for showing the mapped area
        self.model.add_mapping(self.series.pen().color().name(),
                               QRect(2, 0, 2, self.model.rowCount()))

        self.chart.createDefaultAxes()
        self.chart_view = QChartView(self.chart)
        self.chart_view.setRenderHint(QPainter.RenderHint.Antialiasing)
        self.chart_view.setMinimumSize(640, 480)

        # create main layout
        self.main_layout = QGridLayout()
        self.main_layout.addWidget(self.table_view, 1, 0)
        self.main_layout.addWidget(self.chart_view, 1, 1)
        self.main_layout.setColumnStretch(1, 1)
        self.main_layout.setColumnStretch(0, 0)
        self.setLayout(self.main_layout)


if __name__ == "__main__":
    app = QApplication(sys.argv)
    w = TableWidget()
    w.show()
    sys.exit(app.exec())



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR BSD-3-Clause
from __future__ import annotations

from PySide6.QtCore import QSettings, Qt, Slot
from PySide6.QtGui import QIntValidator
from PySide6.QtWidgets import QDialog
from PySide6.QtSerialBus import QCanBus, QCanBusDevice

from ui_connectdialog import Ui_ConnectDialog


class Settings():
    def __init__(self):
        self.plugin_name = ""
        self.device_interface_name = ""
        self.configurations = []
        self.use_configuration_enabled = False
        self.use_model_ring_buffer = True
        self.model_ring_buffer_size = 1000
        self.use_autoscroll = False


class ConnectDialog(QDialog):

    def __init__(self, parent=None):
        super().__init__(parent)
        self.m_ui = Ui_ConnectDialog()
        self.m_currentSettings = Settings()
        self.m_interfaces = []
        self.m_settings = QSettings("QtProject", "CAN example")
        self.m_ui.setupUi(self)

        self.m_ui.errorFilterEdit.setValidator(QIntValidator(0, 0x1FFFFFFF, self))

        self.m_ui.loopbackBox.addItem("unspecified")
        self.m_ui.loopbackBox.addItem("False", False)
        self.m_ui.loopbackBox.addItem("True", True)

        self.m_ui.receiveOwnBox.addItem("unspecified")
        self.m_ui.receiveOwnBox.addItem("False", False)
        self.m_ui.receiveOwnBox.addItem("True", True)

        self.m_ui.canFdBox.addItem("False", False)
        self.m_ui.canFdBox.addItem("True", True)

        self.m_ui.dataBitrateBox.set_flexible_date_rate_enabled(True)

        self.m_ui.okButton.clicked.connect(self.ok)
        self.m_ui.cancelButton.clicked.connect(self.cancel)
        self.m_ui.useConfigurationBox.toggled.connect(self.m_ui.configurationBox.setEnabled)
        self.m_ui.pluginListBox.currentTextChanged.connect(self.plugin_changed)
        self.m_ui.interfaceListBox.currentTextChanged.connect(self.interface_changed)
        self.m_ui.ringBufferBox.checkStateChanged.connect(self._ring_buffer_changed)

        self.m_ui.rawFilterEdit.hide()
        self.m_ui.rawFilterLabel.hide()

        self.m_ui.pluginListBox.addItems(QCanBus.instance().plugins())

        self.restore_settings()

    @Slot(int)
    def _ring_buffer_changed(self, state):
        self.m_ui.ringBufferLimitBox.setEnabled(state == Qt.CheckState.Checked)

    def settings(self):
        return self.m_currentSettings

    def save_settings(self):
        qs = self.m_settings
        cur = self.m_currentSettings
        qs.beginGroup("LastSettings")
        qs.setValue("PluginName", self.m_currentSettings.plugin_name)
        qs.setValue("DeviceInterfaceName", cur.device_interface_name)
        qs.setValue("UseAutoscroll", cur.use_autoscroll)
        qs.setValue("UseRingBuffer", cur.use_model_ring_buffer)
        qs.setValue("RingBufferSize", cur.model_ring_buffer_size)
        qs.setValue("UseCustomConfiguration", cur.use_configuration_enabled)

        if cur.use_configuration_enabled:
            qs.setValue("Loopback",
                        self.configuration_value(QCanBusDevice.LoopbackKey))
            qs.setValue("ReceiveOwn",
                        self.configuration_value(QCanBusDevice.ReceiveOwnKey))
            qs.setValue("ErrorFilter",
                        self.configuration_value(QCanBusDevice.ErrorFilterKey))
            qs.setValue("BitRate",
                        self.configuration_value(QCanBusDevice.BitRateKey))
            qs.setValue("CanFd",
                        self.configuration_value(QCanBusDevice.CanFdKey))
            qs.setValue("DataBitRate",
                        self.configuration_value(QCanBusDevice.DataBitRateKey))
        qs.endGroup()

    def restore_settings(self):
        qs = self.m_settings
        cur = self.m_currentSettings
        qs.beginGroup("LastSettings")
        cur.plugin_name = qs.value("PluginName", "", str)
        cur.device_interface_name = qs.value("DeviceInterfaceName", "", str)
        cur.use_autoscroll = qs.value("UseAutoscroll", False, bool)
        cur.use_model_ring_buffer = qs.value("UseRingBuffer", False, bool)
        cur.model_ring_buffer_size = qs.value("RingBufferSize", 0, int)
        cur.use_configuration_enabled = qs.value("UseCustomConfiguration", False, bool)

        self.revert_settings()

        if cur.use_configuration_enabled:
            self.m_ui.loopbackBox.setCurrentText(qs.value("Loopback"))
            self.m_ui.receiveOwnBox.setCurrentText(qs.value("ReceiveOwn"))
            self.m_ui.errorFilterEdit.setText(qs.value("ErrorFilter"))
            self.m_ui.bitrateBox.setCurrentText(qs.value("BitRate"))
            self.m_ui.canFdBox.setCurrentText(qs.value("CanFd"))
            self.m_ui.dataBitrateBox.setCurrentText(qs.value("DataBitRate"))

        qs.endGroup()
        self.update_settings()

    @Slot(str)
    def plugin_changed(self, plugin):
        self.m_ui.interfaceListBox.clear()
        interfaces, error_string = QCanBus.instance().availableDevices(plugin)
        self.m_interfaces = interfaces
        for info in self.m_interfaces:
            self.m_ui.interfaceListBox.addItem(info.name())

    @Slot(str)
    def interface_changed(self, interface):
        for info in self.m_interfaces:
            if interface == info.name():
                self.m_ui.deviceInfoBox.set_device_info(info)
                return
        self.m_ui.deviceInfoBox.clear()

    @Slot()
    def ok(self):
        self.update_settings()
        self.save_settings()
        self.accept()

    @Slot()
    def cancel(self):
        self.revert_settings()
        self.reject()

    def configuration_value(self, key):
        result = None
        for k, v in self.m_currentSettings.configurations:
            if k == key:
                result = v
                break
        if (not result and (key == QCanBusDevice.ConfigurationKey.LoopbackKey
                            or key == QCanBusDevice.ConfigurationKey.ReceiveOwnKey)):
            return "unspecified"
        return str(result)

    def revert_settings(self):
        self.m_ui.pluginListBox.setCurrentText(self.m_currentSettings.plugin_name)
        self.m_ui.interfaceListBox.setCurrentText(self.m_currentSettings.device_interface_name)
        self.m_ui.useConfigurationBox.setChecked(self.m_currentSettings.use_configuration_enabled)

        self.m_ui.ringBufferBox.setChecked(self.m_currentSettings.use_model_ring_buffer)
        self.m_ui.ringBufferLimitBox.setValue(self.m_currentSettings.model_ring_buffer_size)
        self.m_ui.autoscrollBox.setChecked(self.m_currentSettings.use_autoscroll)

        value = self.configuration_value(QCanBusDevice.ConfigurationKey.LoopbackKey)
        self.m_ui.loopbackBox.setCurrentText(value)

        value = self.configuration_value(QCanBusDevice.ConfigurationKey.ReceiveOwnKey)
        self.m_ui.receiveOwnBox.setCurrentText(value)

        value = self.configuration_value(QCanBusDevice.ConfigurationKey.ErrorFilterKey)
        self.m_ui.errorFilterEdit.setText(value)

        value = self.configuration_value(QCanBusDevice.ConfigurationKey.BitRateKey)
        self.m_ui.bitrateBox.setCurrentText(value)

        value = self.configuration_value(QCanBusDevice.ConfigurationKey.CanFdKey)
        self.m_ui.canFdBox.setCurrentText(value)

        value = self.configuration_value(QCanBusDevice.ConfigurationKey.DataBitRateKey)
        self.m_ui.dataBitrateBox.setCurrentText(value)

    def update_settings(self):
        self.m_currentSettings.plugin_name = self.m_ui.pluginListBox.currentText()
        self.m_currentSettings.device_interface_name = self.m_ui.interfaceListBox.currentText()
        self.m_currentSettings.use_configuration_enabled = self.m_ui.useConfigurationBox.isChecked()

        self.m_currentSettings.use_model_ring_buffer = self.m_ui.ringBufferBox.isChecked()
        self.m_currentSettings.model_ring_buffer_size = self.m_ui.ringBufferLimitBox.value()
        self.m_currentSettings.use_autoscroll = self.m_ui.autoscrollBox.isChecked()

        if self.m_currentSettings.use_configuration_enabled:
            self.m_currentSettings.configurations.clear()
            # process LoopBack
            if self.m_ui.loopbackBox.currentIndex() != 0:
                item = (QCanBusDevice.LoopbackKey, self.m_ui.loopbackBox.currentData())
                self.m_currentSettings.configurations.append(item)

            # process ReceiveOwnKey
            if self.m_ui.receiveOwnBox.currentIndex() != 0:
                item = (QCanBusDevice.ReceiveOwnKey, self.m_ui.receiveOwnBox.currentData())
                self.m_currentSettings.configurations.append(item)

            # process error filter
            error_filter = self.m_ui.errorFilterEdit.text()
            if error_filter:
                ok = False
                try:
                    int(error_filter)  # check if value contains a valid integer
                    ok = True
                except ValueError:
                    pass
                if ok:
                    item = (QCanBusDevice.ErrorFilterKey, error_filter)
                    self.m_currentSettings.configurations.append(item)

            # process raw filter list
            if self.m_ui.rawFilterEdit.text():
                pass  # TODO current ui not sufficient to reflect this param

            # process bitrate
            bitrate = self.m_ui.bitrateBox.bit_rate()
            if bitrate > 0:
                item = (QCanBusDevice.BitRateKey, bitrate)
                self.m_currentSettings.configurations.append(item)

            # process CAN FD setting
            fd_item = (QCanBusDevice.CanFdKey, self.m_ui.canFdBox.currentData())
            self.m_currentSettings.configurations.append(fd_item)

            # process data bitrate
            data_bitrate = self.m_ui.dataBitrateBox.bit_rate()
            if data_bitrate > 0:
                item = (QCanBusDevice.DataBitRateKey, data_bitrate)
                self.m_currentSettings.configurations.append(item)



#!/usr/bin/env python
# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''Test cases for SimpleFile class'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from shiboken_paths import init_paths
init_paths()

from sample import SimpleFile


class SimpleFileTest(unittest.TestCase):
    '''Test cases for SimpleFile class.'''

    def setUp(self):
        filename = f'simplefile{os.getpid()}.txt'
        self.existing_filename = Path(os.curdir) / filename
        self.delete_file = False
        if not self.existing_filename.exists():
            with self.existing_filename.open('w') as f:
                for line in range(10):
                    f.write('sbrubbles\n')
            self.delete_file = True

        self.non_existing_filename = Path(os.curdir) / 'inexistingfile.txt'
        i = 0
        while self.non_existing_filename.exists():
            i += 1
            filename = f'inexistingfile-{i}.txt'
            self.non_existing_filename = Path(os.curdir) / filename

    def tearDown(self):
        if self.delete_file:
            os.remove(self.existing_filename)

    def testExistingFile(self):
        '''Test SimpleFile class with existing file.'''
        f = SimpleFile(os.fspath(self.existing_filename))
        self.assertEqual(f.filename(), os.fspath(self.existing_filename))
        f.open()
        self.assertNotEqual(f.size(), 0)
        f.close()

    def testNonExistingFile(self):
        '''Test SimpleFile class with non-existing file.'''
        f = SimpleFile(os.fspath(self.non_existing_filename))
        self.assertEqual(f.filename(), os.fspath(self.non_existing_filename))
        self.assertRaises(IOError, f.open)
        self.assertEqual(f.size(), 0)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from helper.usesqapplication import UsesQApplication

from PySide6.QtWidgets import QWidget
from PySide6.QtCore import QPoint, QTimer, Qt, QEvent
from PySide6.QtGui import QPointingDevice
from PySide6.QtTest import QTest


class MyWidget(QWidget):
    def __init__(self, parent=None):
        super().__init__(parent)
        self._sequence = []
        self._device = QPointingDevice.primaryPointingDevice()
        self.setAttribute(Qt.WA_AcceptTouchEvents)
        QTimer.singleShot(200, self.generateEvent)

    def event(self, e):
        et = e.type()
        if (et == QEvent.Type.TouchBegin or et == QEvent.Type.TouchUpdate
                or et == QEvent.Type.TouchEnd):
            e.accept()
            self._sequence.append(et)
            return True
        return super().event(e)

    def generateEvent(self):
        QTest.touchEvent(self, self._device).press(0, QPoint(10, 10))
        QTest.touchEvent(self, self._device).stationary(0).press(1, QPoint(40, 10))
        QTest.touchEvent(self, self._device).move(0, QPoint(12, 12)).move(1, QPoint(45, 5))
        QTest.touchEvent(self, self._device).release(0, QPoint(12, 12)).release(1, QPoint(45, 5))
        QTimer.singleShot(200, self.deleteLater)


class TouchEventTest(UsesQApplication):
    @unittest.skipIf(QPointingDevice.primaryPointingDevice() is None, "No device")
    def testCreateEvent(self):
        w = MyWidget()
        w.show()
        self.app.exec()
        self.assertEqual(w._sequence.count(QEvent.Type.TouchBegin), 1)
        self.assertEqual(w._sequence.count(QEvent.Type.TouchUpdate), 2)
        self.assertEqual(w._sequence.count(QEvent.Type.TouchEnd), 1)


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

'''(Very) Simple test case for missing names from QtCore'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6 import QtCore


class MissingClasses(unittest.TestCase):

    def testQSettings(self):  # Bug 232
        getattr(QtCore, 'QSettings')

    def testQtTrNoop(self):  # Bug 220
        getattr(QtCore, 'QT_TR_NOOP')


if __name__ == '__main__':
    unittest.main()



# Copyright (C) 2022 The Qt Company Ltd.
# SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
from __future__ import annotations

''' Test bug 338: http://bugs.openbossa.org/show_bug.cgi?id=338'''

import os
import sys
import unittest

from pathlib import Path
sys.path.append(os.fspath(Path(__file__).resolve().parents[1]))
from init_paths import init_test_paths
init_test_paths(False)

from PySide6.QtWidgets import QApplication, QGraphicsPolygonItem, QGraphicsScene


class DiagramItem(QGraphicsPolygonItem):
    def __init__(self, parent=None, scene=None):
        super().__init__(parent, scene)

    def itemChange(self, change, value):
        return value


class BugTest(unittest.TestCase):
    def test(self):
        app = QApplication(sys.argv)  # noqa: F841
        scene = QGraphicsScene()
        item = DiagramItem()
        item2 = DiagramItem()
        # this cause segfault
        scene.addItem(item)
        scene.addItem(item2)


if __name__ == "__main__":
    unittest.main()



# Resource object code (Python 3)
# Created by: object code
# Created by: The Resource Compiler for Qt version 6.2.2
# WARNING! All changes made in this file will be lost!

from PySide6 import QtCore

qt_resource_data = b"\
\x00\x00\x00\xf7\
Q\
t\x0aQuarterly\x0ais\x0aa\
\x0apaper\x0abased\x0anew\
sletter\x0aexclusiv\
ely\x0aavailable\x0ato\
\x0aQt\x0acustomers\x0aEv\
ery\x0aquarter\x0awe\x0am\
ail\x0aout\x0aan\x0aissue\
\x0athat\x0awe\x0ahope\x0awi\
ll\x0abring\x0aadded\x0ai\
nsight\x0aand\x0apleas\
ure\x0ato\x0ayour\x0aQt\x0ap\
rogramming\x0awith\x0a\
high\x0aquality\x0atec\
hnical\x0aarticles\x0a\
written\x0aby\x0aQt\x0aex\
perts\x0a\
"

qt_resource_name = b"\
\x00\x0a\
\x0b\x0b\x17\xd9\
\x00d\
\x00i\x00c\x00t\x00i\x00o\x00n\x00a\x00r\x00y\
\x00\x09\
\x08\xb6\xa74\
\x00w\
\x00o\x00r\x00d\x00s\x00.\x00t\x00x\x00t\
"

qt_resource_struct = b"\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x01\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x00\x00\x02\x00\x00\x00\x01\x00\x00\x00\x02\
\x00\x00\x00\x00\x00\x00\x00\x00\
\x00\x00\x00\x1a\x00\x00\x00\x00\x00\x01\x00\x00\x00\x00\
\x00\x00\x01z\xe7\xee'\x05\
"

def qInitResources():
    QtCore.qRegisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

def qCleanupResources():
    QtCore.qUnregisterResourceData(0x03, qt_resource_struct, qt_resource_name, qt_resource_data)

qInitResources()
